@@ -1,5 +1,3 @@
-export { AccountsMonitor };
-
 import Gio from 'gi://Gio';
 import GObject from 'gi://GObject';
 import Polari from 'gi://Polari';
@@ -7,13 +5,13 @@ import Tp from 'gi://TelepathyGLib';
 
 const Signals = imports.signals;
 
-import { NetworksManager } from './networksManager.js';
+import NetworksManager from './networksManager.js';
 
 Gio._promisify(Tp.AccountManager.prototype, 'prepare_async', 'prepare_finish');
 Gio._promisify(Tp.Account.prototype,
     'request_presence_async', 'request_presence_finish');
 
-const AccountsMonitor = class {
+export default class AccountsMonitor {
     static getDefault() {
         if (!this._singleton)
             this._singleton = new AccountsMonitor();
@@ -213,7 +211,7 @@ const AccountsMonitor = class {
             });
         });
     }
-};
+}
 Signals.addSignalMethods(AccountsMonitor.prototype);
 
 const ClientFactory = GObject.registerClass(
