@@ -1,13 +1,3 @@
-export {
-    MessageNotification,
-    UndoNotification,
-    NotificationQueue,
-    SimpleOutput,
-    GridOutput,
-    CommandOutputQueue,
-    MessageInfoBar
-};
-
 import GLib from 'gi://GLib';
 import GObject from 'gi://GObject';
 import Gtk from 'gi://Gtk';
@@ -34,11 +24,11 @@ const AppNotification = GObject.registerClass({
 
     _onChildRevealed() {
         if (!this.child_revealed)
-            this.destroy();
+            this.hide();
     }
 });
 
-const MessageNotification = GObject.registerClass(
+export const MessageNotification = GObject.registerClass(
 class MessageNotification extends AppNotification {
     _init(label, iconName) {
         super._init();
@@ -48,37 +38,38 @@ class MessageNotification extends AppNotification {
         this._box = new Gtk.Box({ spacing: 12 });
 
         if (iconName)
-            this._box.add(new Gtk.Image({ icon_name: iconName }));
+            this._box.append(new Gtk.Image({ icon_name: iconName }));
 
-        this._box.add(new Gtk.Label({
+        this._box.append(new Gtk.Label({
             label,
             hexpand: true,
             ellipsize: Pango.EllipsizeMode.END,
         }));
 
-        let closeButton = new Gtk.Button({ relief: Gtk.ReliefStyle.NONE });
-        closeButton.image = new Gtk.Image({ icon_name: 'window-close-symbolic' });
+        let closeButton = new Gtk.Button({
+            icon_name: 'window-close-symbolic',
+            has_frame: false,
+        });
         closeButton.connect('clicked', this.close.bind(this));
         this._box.pack_end(closeButton, false, false, 0);
 
-        this.add(this._box);
-        this.show_all();
+        this.set_child(this._box);
     }
 
 
     addButton(label, callback) {
-        let button = new Gtk.Button({ label, visible: true });
+        let button = new Gtk.Button({ label });
         button.connect('clicked', () => {
             if (callback)
                 callback();
             this.close();
         });
 
-        this._box.add(button);
+        this._box.append(button);
     }
 });
 
-const UndoNotification = GObject.registerClass({
+export const UndoNotification = GObject.registerClass({
     Signals: {
         closed: {},
         undo: {},
@@ -119,7 +110,7 @@ const CommandOutputNotification = GObject.registerClass({
     }
 });
 
-const SimpleOutput = GObject.registerClass(
+export const SimpleOutput = GObject.registerClass(
 class SimpleOutput extends CommandOutputNotification {
     _init(text) {
         super._init();
@@ -127,15 +118,13 @@ class SimpleOutput extends CommandOutputNotification {
         let label = new Gtk.Label({
             label: text,
             vexpand: true,
-            visible: true,
             wrap: true,
         });
-        this.add(label);
-        this.show_all();
+        this.set_child(label);
     }
 });
 
-const GridOutput = GObject.registerClass(
+export const GridOutput = GObject.registerClass(
 class GridOutput extends CommandOutputNotification {
     _init(header, items) {
         super._init();
@@ -162,53 +151,60 @@ class GridOutput extends CommandOutputNotification {
             }
             row++;
         }
-        this.add(grid);
-        this.show_all();
+        this.set_child(grid);
     }
 });
 
-const NotificationQueue = GObject.registerClass(
+export const NotificationQueue = GObject.registerClass(
 class NotificationQueue extends Gtk.Frame {
     _init() {
         super._init({
             valign: Gtk.Align.START,
             halign: Gtk.Align.CENTER,
             margin_start: 24, margin_end: 24,
-            no_show_all: true,
+            visible: false,
         });
-        this.get_style_context().add_class('app-notification');
+        this.add_css_class('app-notification');
 
         this._grid = new Gtk.Grid({
             orientation: Gtk.Orientation.VERTICAL,
-            row_spacing: 6, visible: true,
+            row_spacing: 6,
         });
-        this.add(this._grid);
+        this.set_child(this._grid);
     }
 
     addNotification(notification) {
-        this._grid.add(notification);
+        this._grid.attach_next_to(notification,
+            null, Gtk.PositionType.BOTTOM, 1, 1);
 
-        notification.connect('destroy', this._onChildDestroy.bind(this));
+        notification.connect('notify::visible',
+            this._onChildVisibleChanged.bind(this));
         this.show();
     }
 
-    _onChildDestroy() {
-        if (this._grid.get_children().length === 0)
+    _onChildVisibleChanged(child) {
+        if (child.visible)
+            return;
+
+        this._grid.remove(child);
+        child.run_dispose();
+
+        if (this._grid.get_first_child() === null)
             this.hide();
     }
 });
 
-const CommandOutputQueue = GObject.registerClass(
+export const CommandOutputQueue = GObject.registerClass(
 class CommandOutputQueue extends NotificationQueue {
     _init() {
         super._init();
 
         this.valign = Gtk.Align.END;
-        this.get_style_context().add_class('irc-feedback');
+        this.add_css_class('irc-feedback');
     }
 });
 
-const MessageInfoBar = GObject.registerClass({
+export const MessageInfoBar = GObject.registerClass({
     Properties: {
         'title': GObject.ParamSpec.string(
             'title', 'title', 'title',
@@ -232,7 +228,7 @@ const MessageInfoBar = GObject.registerClass({
         super._init(Object.assign(defaultParams, params));
 
         let box = new Gtk.Box({ orientation: Gtk.Orientation.VERTICAL });
-        this.get_content_area().add(box);
+        this.add_child(box);
 
         this._titleLabel = new Gtk.Label({
             halign: Gtk.Align.START,
@@ -241,7 +237,7 @@ const MessageInfoBar = GObject.registerClass({
             use_markup: true,
             wrap: true,
         });
-        box.add(this._titleLabel);
+        box.append(this._titleLabel);
 
         this._subtitleLabel = new Gtk.Label({
             halign: Gtk.Align.START,
@@ -249,11 +245,9 @@ const MessageInfoBar = GObject.registerClass({
             label: this._subtitle,
             ellipsize: Pango.EllipsizeMode.END,
         });
-        box.add(this._subtitleLabel);
+        box.append(this._subtitleLabel);
 
         this.connect('response', () => (this.revealed = false));
-
-        box.show_all();
     }
 
     get title() {
