@@ -1,24 +1,24 @@
-export { Application };
-
-import Gdk from 'gi://Gdk?version=3.0';
+import Gdk from 'gi://Gdk?version=4.0';
 import Gio from 'gi://Gio';
 import GLib from 'gi://GLib';
 import GObject from 'gi://GObject';
-import Gtk from 'gi://Gtk?version=3.0';
+import Gtk from 'gi://Gtk?version=4.0';
 import Polari from 'gi://Polari';
 import Tp from 'gi://TelepathyGLib';
 
-import { AccountsMonitor } from './accountsMonitor.js';
+import { setConsoleLogDomain } from 'console';
+
+import AccountsMonitor from './accountsMonitor.js';
 import * as AppNotifications from './appNotifications.js';
 import * as Connections from './connections.js';
-import { InitialSetupWindow } from './initialSetup.js';
-import { MainWindow } from './mainWindow.js';
-import { NetworksManager } from './networksManager.js';
-import { PasteManager } from './pasteManager.js';
-import { RoomManager } from './roomManager.js';
-import { ServerRoomManager } from './serverRoomManager.js';
-import { TelepathyClient } from './telepathyClient.js';
-import { UserStatusMonitor } from './userTracker.js';
+import InitialSetupWindow from './initialSetup.js';
+import MainWindow from './mainWindow.js';
+import NetworksManager from './networksManager.js';
+import PasteManager from './pasteManager.js';
+import RoomManager from './roomManager.js';
+import ServerRoomManager from './serverRoomManager.js';
+import TelepathyClient from './telepathyClient.js';
+import UserStatusMonitor from './userTracker.js';
 import * as Utils from './utils.js';
 
 Gio._promisify(Tp.AccountRequest.prototype,
@@ -37,7 +37,7 @@ const MAX_RETRIES = 3;
 
 const IRC_SCHEMA_REGEX = /^(irc?:\/\/)([\da-z.-]+):?(\d+)?\/(?:%23)?([\w.+-]+)/i;
 
-const Application = GObject.registerClass({
+export default GObject.registerClass({
     Signals: {
         'prepare-shutdown': {},
         'room-focus-changed': {},
@@ -51,11 +51,18 @@ const Application = GObject.registerClass({
 
         GLib.set_prgname('polari');
         Tp.debug_set_flags(GLib.getenv('TP_DEBUG') || '');
+
+        const logDomain = 'Polari';
+        setConsoleLogDomain(logDomain);
+        if (GLib.log_writer_is_journald(2))
+            GLib.setenv('G_MESSAGES_DEBUG', logDomain, false);
+
         this._retryData = new Map();
         this._nickTrackData = new Map();
         this._demons = [];
 
-        this._windowRemovedId = 0;
+        this._windowRemovedId =
+            this.connect('window-removed', this._onWindowRemoved.bind(this));
 
         this.add_main_option('start-client', 0,
             GLib.OptionFlags.NONE, GLib.OptionArg.NONE,
@@ -133,12 +140,13 @@ const Application = GObject.registerClass({
     }
 
     _ensureService(conn, name, opath, iface, command) {
-        debug(`Trying to ensure service ${name}`);
+        console.info(`Trying to ensure service ${name}`);
 
         if (this._checkService(conn, name, opath, iface))
             return;
 
-        log(`Failed to activate service ${name}, starting manually`);
+        console.info(`Failed to activate service ${
+            name}, starting manually`);
 
         let proc = new Gio.Subprocess({ argv: [command] });
 
@@ -146,7 +154,9 @@ const Application = GObject.registerClass({
             proc.init(null);
             this._demons.push(proc);
         } catch (e) {
-            log('Failed to launch %s: %s'.format(command, e.message));
+            console.error(`Failed to launch ${
+                command} to provide service ${name}`);
+            console.debug(e);
         }
     }
 
@@ -157,11 +167,12 @@ const Application = GObject.registerClass({
 
         let handled = false;
         let id = Gio.bus_watch_name(bus, name, flags, () => {
-            debug('Running as test instance alongside primary instance');
+            console.info(
+                'Running as test instance alongside primary instance');
             this.set_flags(this.flags | Gio.ApplicationFlags.NON_UNIQUE);
             handled = true;
         }, () => {
-            debug('No primary instance found, running normally');
+            console.info('No primary instance found, running normally');
             handled = true;
         });
 
@@ -211,11 +222,11 @@ const Application = GObject.registerClass({
         }, {
             name: 'join-room',
             activate: this._onJoinRoom.bind(this),
-            parameter_type: GLib.VariantType.new('(ssu)'),
+            parameter_type: GLib.VariantType.new('(ssb)'),
         }, {
             name: 'message-user',
             activate: this._onMessageUser.bind(this),
-            parameter_type: GLib.VariantType.new('(sssu)'),
+            parameter_type: GLib.VariantType.new('(sssb)'),
         }, {
             name: 'leave-room',
             parameter_type: GLib.VariantType.new('(ss)'),
@@ -377,17 +388,11 @@ const Application = GObject.registerClass({
         });
 
         this.pasteManager = new PasteManager();
-        this.notificationQueue = new AppNotifications.NotificationQueue();
-        this.commandOutputQueue = new AppNotifications.CommandOutputQueue();
 
         let provider = new Gtk.CssProvider();
         let uri = 'resource:///org/gnome/Polari/css/application.css';
-        try {
-            provider.load_from_file(Gio.File.new_for_uri(uri));
-        } catch (e) {
-            logError(e, 'Failed to add application style');
-        }
-        Gtk.StyleContext.add_provider_for_screen(Gdk.Screen.get_default(),
+        provider.load_from_file(Gio.File.new_for_uri(uri));
+        Gtk.StyleContext.add_provider_for_display(Gdk.Display.get_default(),
             provider,
             Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION);
     }
@@ -396,22 +401,10 @@ const Application = GObject.registerClass({
         this.activate_action('start-client', null);
 
         if (!this.active_window) {
-            if (this._windowRemovedId)
-                this.disconnect(this._windowRemovedId);
-            this._windowRemovedId = 0;
-
             if (this._needsInitialSetup()) {
                 new InitialSetupWindow({ application: this });
-                this._windowRemovedId = this.connect('window-removed', () => {
-                    this.activate();
-                });
             } else {
                 let window = new MainWindow({ application: this });
-                this._windowRemovedId = this.connect('window-removed', () => {
-                    if (this._settings.get_boolean('run-in-background'))
-                        return;
-                    this.emit('prepare-shutdown');
-                });
                 window.connect('notify::active-room',
                     () => this.emit('room-focus-changed'));
                 window.connect('notify::is-active',
@@ -429,28 +422,40 @@ const Application = GObject.registerClass({
             return;
 
         let action = this.lookup_action('leave-current-room');
-        window.connect('notify::active-room', () => {
-            action.enabled = window.active_room !== null;
-        });
         action.enabled = window.active_room !== null;
 
-        window.connect('active-room-state-changed',
-            this._updateUserListAction.bind(this));
+        this._toplevelSignals = [
+            window.connect('notify::active-room',
+                () => (action.enabled = window.active_room !== null)),
+            window.connect('active-room-state-changed',
+                () => this._updateUserListAction()),
+        ];
         this._updateUserListAction();
     }
 
+    _onWindowRemoved(app, window) {
+        if (!(window instanceof MainWindow))
+            this.activate();
+        else if (!this._settings.get_boolean('run-in-background'))
+            this.emit('prepare-shutdown');
+
+        this._toplevelSignals.forEach(id => window.disconnect(id));
+        this._toplevelSignals = [];
+
+        window.run_dispose();
+    }
+
     vfunc_open(files) {
         this.activate();
 
-        let time = Utils.getTpEventTime();
         let uris = files.map(f => f.get_uri());
 
         this._accountsMonitor.prepare(() => {
-            this._openURIs(uris, time);
+            this._openURIs(uris);
         });
     }
 
-    _openURIs(uris, time) {
+    _openURIs(uris) {
         let map = {};
 
         this._accountsMonitor.visibleAccounts.forEach(a => {
@@ -482,7 +487,7 @@ const Application = GObject.registerClass({
                 accountPath = account.get_object_path();
             }
 
-            joinAction.activate(new GLib.Variant('(ssu)', [accountPath, `#${room}`, time]));
+            joinAction.activate(new GLib.Variant('(ssb)', [accountPath, `#${room}`, true]));
         });
     }
 
@@ -496,7 +501,7 @@ const Application = GObject.registerClass({
             let label = _('Failed to open link');
             let n = new AppNotifications.MessageNotification(
                 label, 'dialog-error-symbolic');
-            this.notificationQueue.addNotification(n);
+            this.active_window?.queueNotification(n);
         }
 
         return [success, server, port, room];
@@ -575,21 +580,19 @@ const Application = GObject.registerClass({
         this.active_window.showJoinRoomDialog();
     }
 
-    _maybePresent(time) {
-        let [present] = Tp.user_action_time_should_present(time);
-
+    _maybePresent(present) {
         if (!this.active_window || present)
             this.activate();
     }
 
     _onJoinRoom(action, parameter) {
-        let [accountPath_, channelName_, time] = parameter.deep_unpack();
-        this._maybePresent(time);
+        let [accountPath_, channelName_, present] = parameter.deep_unpack();
+        this._maybePresent(present);
     }
 
     _onMessageUser(action, parameter) {
-        let [accountPath_, contactName_, msg_, time] = parameter.deep_unpack();
-        this._maybePresent(time);
+        let [accountPath_, contactName_, msg_, present] = parameter.deep_unpack();
+        this._maybePresent(present);
     }
 
     _trackNominalNick(account) {
@@ -631,7 +634,8 @@ const Application = GObject.registerClass({
 
         let params = account.getConnectionParams();
         let { server, account: accountName, port } = params;
-        debug(`Failed to connect to ${server} with username ${accountName}`);
+        console.info(`Failed to connect to ${server} with username ${
+            accountName}`);
 
         let accountServers = [];
         if (account.predefined)
@@ -677,7 +681,7 @@ const Application = GObject.registerClass({
         let oldParams = account.dup_parameters_vardict().deep_unpack();
         let nick = oldParams['account'].deep_unpack();
 
-        debug(`Retrying with nickname ${nick}_`);
+        console.info(`Retrying with nickname ${nick}_`);
         let params = { account: new GLib.Variant('s', `${nick}_`) };
         this._retryWithParams(account, new GLib.Variant('a{sv}', params));
         return true;
@@ -690,7 +694,7 @@ const Application = GObject.registerClass({
         if (!server)
             return false;
 
-        debug(`Retrying with ${server.address}:${server.port}`);
+        console.info(`Retrying with ${server.address}:${server.port}`);
         let params = {
             server: new GLib.Variant('s', server.address),
             port: new GLib.Variant('u', server.port),
@@ -723,7 +727,8 @@ const Application = GObject.registerClass({
             if (reason !== Tp.ConnectionStatusReason.REQUESTED) {
                 let strReason = Object.keys(Tp.ConnectionStatusReason)[reason];
                 let name = account.display_name;
-                debug(`Account ${name} disconnected with reason ${strReason}`);
+                console.info(`Account ${name} disconnected with reason ${
+                    strReason}`);
 
                 // Connection failed, keep tp from retrying over and over
                 let presence = Tp.ConnectionPresenceType.OFFLINE;
@@ -763,9 +768,9 @@ const Application = GObject.registerClass({
         await account.set_enabled_async(false);
         account.visible = false;
 
-        const label = _('%s removed.').format(account.display_name);
+        const label = vprintf(_('%s removed.'), account.display_name);
         const n = new AppNotifications.UndoNotification(label);
-        this.notificationQueue.addNotification(n);
+        this?.active_window.queueNotification(n);
 
         n.connect('closed', async () => {
             await account.remove_async(); // TODO: Check for errors
@@ -803,7 +808,7 @@ const Application = GObject.registerClass({
     }
 
     _onShowHelp() {
-        Utils.openURL('help:polari', Gtk.get_current_event_time());
+        Utils.openURL('help:polari');
     }
 
     _onShowAbout() {
