@@ -1,20 +1,18 @@
-export { ChatView };
-
 import Gdk from 'gi://Gdk';
 import Gio from 'gi://Gio';
 import GLib from 'gi://GLib';
 import GObject from 'gi://GObject';
+import Graphene from 'gi://Graphene';
 import Gtk from 'gi://Gtk';
 import Pango from 'gi://Pango';
-import PangoCairo from 'gi://PangoCairo';
 import Polari from 'gi://Polari';
 import Tp from 'gi://TelepathyGLib';
 import Tpl from 'gi://TelepathyLogger';
 
 import { DropTargetIface } from './pasteManager.js';
 import { UserPopover } from './userList.js';
-import { UserStatusMonitor } from './userTracker.js';
-import { URLPreview } from './urlPreview.js';
+import UserStatusMonitor from './userTracker.js';
+import URLPreview from './urlPreview.js';
 import * as Utils from './utils.js';
 
 Gio._promisify(Tpl.LogWalker.prototype,
@@ -46,46 +44,105 @@ const NICK_SPACING = 14;
 
 const NICKTAG_PREFIX = 'nick';
 
-function _getColor(context) {
-    let color = context.get_color(context.get_state());
-    color.alpha *= context.get_property('opacity', context.get_state());
-    return color;
-}
-
 // Workaround for GtkTextView growing horizontally over time when
 // added to a GtkScrolledWindow with horizontal scrolling disabled
-const TextView = GObject.registerClass(
-class TextView extends Gtk.TextView {
+const TextView = GObject.registerClass({
+    Properties: {
+        'indent-width-chars': GObject.ParamSpec.uint(
+            'indent-width-chars', 'indent-width-chars', 'indent-width-chars',
+            GObject.ParamFlags.READWRITE,
+            0, GLib.MAXUINT32, 0),
+        'indext-spacing': GObject.ParamSpec.uint(
+            'indent-spacing', 'indent-spacing', 'indent-spacing',
+            GObject.ParamFlags.READWRITE,
+            0, GLib.MAXUINT32, 0),
+    },
+}, class TextView extends Gtk.TextView {
     _init(params) {
+        this._indentWidthChars = 0;
+        this._indentSpacing = 0;
+
         super._init(params);
 
         this.buffer.connect('mark-set', this._onMarkSet.bind(this));
-        this.connect('screen-changed', this._updateLayout.bind(this));
+        this.connect('notify::root', this._onScreenChanged.bind(this));
     }
 
-    vfunc_get_preferred_width() {
-        return [1, 1];
+    // eslint-disable-next-line camelcase
+    get indent_width_chars() {
+        return this._indentWidthChars;
     }
 
-    vfunc_style_updated() {
-        let context = this.get_style_context();
-        context.save();
-        context.add_class('dim-label');
-        context.set_state(Gtk.StateFlags.NORMAL);
-        this._dimColor = _getColor(context);
-        context.restore();
+    // eslint-disable-next-line camelcase
+    set indent_width_chars(value) {
+        if (this._indentWidthChars === value)
+            return;
 
-        super.vfunc_style_updated();
+        this._indentWidthChars = value;
+        this.notify('indent-width-chars');
+
+        this._updateIndent();
     }
 
-    vfunc_draw(cr) {
-        super.vfunc_draw(cr);
+    // eslint-disable-next-line camelcase
+    get indent_spacing() {
+        return this._indentSpacing;
+    }
+
+    // eslint-disable-next-line camelcase
+    set indent_spacing(value) {
+        if (this._indentSpacing === value)
+            return;
+
+        this._indentSpacing = value;
+        this.notify('indent-spacing');
+        this._updateIndent();
+    }
+
+    _updateIndent() {
+        const context = this.get_pango_context();
+        const metrics = context.get_metrics(null, null);
+        const charWidth = Math.max(
+            metrics.get_approximate_char_width(),
+            metrics.get_approximate_digit_width());
+        const pixelWidth = Pango.units_to_double(charWidth);
+
+        const totalWidth =
+            this._indentWidthChars * pixelWidth + this._indentSpacing;
+
+        const tabs = Pango.TabArray.new(1, true);
+        tabs.set_tab(0, Pango.TabAlign.LEFT, totalWidth);
+
+        this.set({
+            tabs,
+            indent: -totalWidth,
+        });
+    }
+
+    vfunc_measure(orientation, forSize) {
+        const [min, nat] = orientation === Gtk.Orientation.HORIZONTAL
+            ? [1, 1] : super.vfunc_measure(orientation, forSize);
+        return [min, nat, -1, -1];
+    }
+
+    vfunc_css_changed(change) {
+        super.vfunc_css_changed(change);
+
+        const context = this.get_style_context();
+        [, this._dimColor] = context.lookup_color('inactive_nick_color');
+
+        /* pick up DPI changes (e.g. via the 'text-scaling-factor' setting):
+           the default handler calls pango_cairo_context_set_resolution(), so
+           update the indent after that */
+        this._updateIndent();
+    }
+
+    vfunc_snapshot(snapshot) {
+        super.vfunc_snapshot(snapshot);
 
         let mark = this.buffer.get_mark('indicator-line');
-        if (!mark) {
-            cr.$dispose();
-            return Gdk.EVENT_PROPAGATE;
-        }
+        if (!mark)
+            return;
 
         let iter = this.buffer.get_iter_at_mark(mark);
         let location = this.get_iter_location(iter);
@@ -112,28 +169,29 @@ class TextView extends Gtk.TextView {
         let baseline = Math.floor(this._layout.get_baseline() / Pango.SCALE);
         let layoutY = y - baseline + Math.floor((layoutHeight - baseline) / 2) + 0.5;
 
-        let [hasClip, clip] = Gdk.cairo_get_clip_rectangle(cr);
-        if (hasClip &&
-            clip.y <= layoutY + layoutHeight &&
-            clip.y + clip.height >= layoutY) {
-            Gdk.cairo_set_source_rgba(cr, this._dimColor);
+        const bounds = new Graphene.Rect();
+        bounds.init(MARGIN, y, width, 1);
 
-            cr.moveTo(layoutX, layoutY);
-            PangoCairo.show_layout(cr, this._layout);
+        snapshot.save();
 
-            let [, color] = this.get_style_context().lookup_color('borders');
-            Gdk.cairo_set_source_rgba(cr, color);
+        snapshot.translate(new Graphene.Point({ x: layoutX, y: layoutY }));
+        snapshot.append_layout(this._layout, this._dimColor);
+
+        snapshot.restore();
+
+        const cr = snapshot.append_cairo(bounds);
+
+        const [, color] = this.get_style_context().lookup_color('borders');
+        Gdk.cairo_set_source_rgba(cr, color);
+
+        cr.setLineWidth(1);
+        cr.moveTo(MARGIN, y);
+        cr.lineTo(layoutX - MARGIN, y);
+        cr.moveTo(layoutX + layoutWidth + MARGIN, y);
+        cr.lineTo(MARGIN + width, y);
+        cr.stroke();
 
-            cr.setLineWidth(1);
-            cr.moveTo(MARGIN, y);
-            cr.lineTo(layoutX - MARGIN, y);
-            cr.moveTo(layoutX + layoutWidth + MARGIN, y);
-            cr.lineTo(MARGIN + width, y);
-            cr.stroke();
-        }
         cr.$dispose();
-
-        return Gdk.EVENT_PROPAGATE;
     }
 
     _onMarkSet(buffer, iter, mark) {
@@ -141,9 +199,11 @@ class TextView extends Gtk.TextView {
             this.queue_draw();
     }
 
-    _updateLayout() {
+    _onScreenChanged() {
         this._layout = this.create_pango_layout(null);
         this._layout.set_markup(`<small><b>${_('New Messages')}</b></small>`, -1);
+
+        this._updateIndent();
     }
 });
 
@@ -155,15 +215,12 @@ const ButtonTag = GObject.registerClass({
             false),
     },
     Signals: {
-        'clicked': {},
-        'popup-menu': {},
+        'clicked': { param_types: [GObject.TYPE_DOUBLE, GObject.TYPE_DOUBLE] },
+        'popup-menu': { param_types: [GObject.TYPE_DOUBLE, GObject.TYPE_DOUBLE] },
     },
 }, class ButtonTag extends Gtk.TextTag {
     _init(params) {
         this._hover = false;
-
-        this._gesture = null;
-
         super._init(params);
     }
 
@@ -179,42 +236,12 @@ const ButtonTag = GObject.registerClass({
         this.notify('hover');
     }
 
-    vfunc_event(object, event, _iter) {
-        this._ensureGesture(object);
-
-        if (this._gesture.handle_event(event) ||
-            this._gesture.is_recognized())
-            return Gdk.EVENT_STOP;
-
-        return Gdk.EVENT_PROPAGATE;
+    clicked(...coords) {
+        this.emit('clicked', ...coords);
     }
 
-    _ensureGesture(widget) {
-        if (this._gesture)
-            return;
-
-        this._gesture = new Gtk.GestureMultiPress({
-            widget,
-            button: 0,
-            exclusive: true,
-        });
-
-        this._gesture.connect('pressed', (gesture, nPress) => {
-            if (!this._hover || nPress > 1)
-                return;
-
-            let button = this._gesture.get_current_button();
-            if (button === Gdk.BUTTON_SECONDARY)
-                this.emit('popup-menu');
-        });
-        this._gesture.connect('released', (gesture, nPress) => {
-            if (!this._hover || nPress > 1)
-                return;
-
-            let button = this._gesture.get_current_button();
-            if (button === Gdk.BUTTON_PRIMARY)
-                this.emit('clicked');
-        });
+    popupMenu(...coords) {
+        this.emit('popup-menu', ...coords);
     }
 });
 
@@ -282,7 +309,7 @@ const HoverFilterTag = GObject.registerClass({
     }
 });
 
-const ChatView = GObject.registerClass({
+export default GObject.registerClass({
     Implements: [DropTargetIface],
     Properties: {
         'can-drop': GObject.ParamSpec.override('can-drop', DropTargetIface),
@@ -295,26 +322,47 @@ const ChatView = GObject.registerClass({
     _init(room) {
         super._init({ hscrollbar_policy: Gtk.PolicyType.NEVER, vexpand: true });
 
-        this.get_style_context().add_class('polari-chat-view');
+        this.add_css_class('polari-chat-view');
+
+        this._actionGroup = new Gio.SimpleActionGroup();
+        this.insert_action_group('chatlog', this._actionGroup);
+
+        let action;
+        action = new Gio.SimpleAction({
+            name: 'open-url',
+            parameter_type: new GLib.VariantType('s'),
+        });
+        action.connect('activate',
+            (a, params) => Utils.openURL(params.unpack()));
+        this._actionGroup.add_action(action);
+
+        action = new Gio.SimpleAction({
+            name: 'copy-url',
+            parameter_type: new GLib.VariantType('s'),
+        });
+        action.connect('activate', (a, params) => {
+            const clipboard = this.get_clipboard();
+            clipboard.set(params.unpack());
+        });
+        this._actionGroup.add_action(action);
 
         this._view = new TextView({
             editable: false, cursor_visible: false,
             wrap_mode: Gtk.WrapMode.WORD_CHAR,
             right_margin: MARGIN,
+            indent_width_chars: MAX_NICK_CHARS,
+            indent_spacing: NICK_SPACING,
         });
-        this._view.add_events(Gdk.EventMask.LEAVE_NOTIFY_MASK |
-                              Gdk.EventMask.ENTER_NOTIFY_MASK);
-        this.add(this._view);
-        this.show_all();
+        this._view.bind_property_full('indent',
+            this._view, 'left-margin',
+            GObject.BindingFlags.SYNC_CREATE,
+            (v, source) => [true, MARGIN - source],
+            null);
+        this.set_child(this._view);
 
         this._createTags();
 
-        this.connect('style-updated', this._onStyleUpdated.bind(this));
-        this._onStyleUpdated();
-
         this.connect('destroy', this._onDestroy.bind(this));
-        this.connect('screen-changed', this._updateIndent.bind(this));
-        this.connect('scroll-event', this._onScroll.bind(this));
         this.connect('edge-reached', (w, pos) => {
             if (pos === Gtk.PositionType.BOTTOM)
                 this._autoscroll = true;
@@ -326,23 +374,38 @@ const ChatView = GObject.registerClass({
         this.vadjustment.connect('notify::upper',
             this._onUpperChanged.bind(this));
 
-        this._view.connect('key-press-event', this._onKeyPress.bind(this));
-        this._view.connect('motion-notify-event',
+        this._scrollController = new Gtk.EventControllerScroll({
+            flags: Gtk.EventControllerScrollFlags.VERTICAL,
+        });
+        this._scrollController.connect('scroll', this._onScroll.bind(this));
+        this.add_controller(this._scrollController);
+
+        this._keyController = new Gtk.EventControllerKey();
+        this._keyController.connect('key-pressed', this._onKeyPressed.bind(this));
+        this._view.add_controller(this._keyController);
+
+        this._motionController = new Gtk.EventControllerMotion();
+        this._motionController.connect('motion',
             this._handleButtonTagsHover.bind(this));
-        this._view.connect('enter-notify-event',
+        this._motionController.connect('enter',
             this._handleButtonTagsHover.bind(this));
-        this._view.connect('leave-notify-event',
+        this._motionController.connect('leave',
             this._handleButtonTagsHover.bind(this));
-        /* pick up DPI changes (e.g. via the 'text-scaling-factor' setting):
-           the default handler calls pango_cairo_context_set_resolution(), so
-           update the indent after that */
-        this._view.connect_after('style-updated',
-            this._updateIndent.bind(this));
+        this._view.add_controller(this._motionController);
+
+        this._clickGesture = new Gtk.GestureClick({
+            propagation_phase: Gtk.PropagationPhase.CAPTURE,
+            button: 0,
+        });
+        this._clickGesture.connect('pressed',
+            this._handleButtonTagPressed.bind(this));
+        this._clickGesture.connect('released',
+            this._handleButtonTagReleased.bind(this));
+        this._view.add_controller(this._clickGesture);
 
         this._room = room;
         this._state = { lastNick: null, lastTimestamp: 0, lastStatusGroup: 0 };
         this._joinTime = 0;
-        this._maxNickChars = MAX_NICK_CHARS;
         this._hoveredButtonTags = [];
         this._needsIndicator = true;
         this._pending = new Map();
@@ -351,6 +414,11 @@ const ChatView = GObject.registerClass({
         this._backlogTimeoutId = 0;
         this._statusCount = { left: 0, joined: 0, total: 0 };
 
+        this._activeNickColor = new Gdk.RGBA();
+        this._inactiveNickColor = new Gdk.RGBA();
+        this._hoveredLinkColor = new Gdk.RGBA();
+        this._statusHeaderHoverColor = new Gdk.RGBA();
+
         let statusMonitor = UserStatusMonitor.getDefault();
         this._userTracker = statusMonitor.getUserTrackerForAccount(room.account);
 
@@ -376,14 +444,11 @@ const ChatView = GObject.registerClass({
         this._originalUpper = this.vadjustment.get_upper();
 
         this._app = Gio.Application.get_default();
-        DropTargetIface.addTargets(this, this._view);
+        this.addTargets(this._view);
 
         this._roomFocusChangedId = this._app.connect('room-focus-changed',
             this._checkMessages.bind(this));
 
-        this._hoverCursor = Gdk.Cursor.new_from_name(
-            this.get_display(), 'pointer');
-
         this._channelSignals = [];
         this._channel = null;
 
@@ -439,7 +504,6 @@ const ChatView = GObject.registerClass({
             name: 'status',
             left_margin: MARGIN,
             indent: 0,
-            justification: Gtk.Justification.RIGHT,
         }, {
             name: 'timestamp',
             left_margin: MARGIN,
@@ -463,53 +527,35 @@ const ChatView = GObject.registerClass({
         tags.forEach(tagProps => tagTable.add(new Gtk.TextTag(tagProps)));
     }
 
-    _onStyleUpdated() {
-        let context = this.get_style_context();
-        context.save();
-        context.add_class('dim-label');
-        context.set_state(Gtk.StateFlags.NORMAL);
-        let dimColor = _getColor(context);
-        context.restore();
+    vfunc_css_changed(change) {
+        super.vfunc_css_changed(change);
 
-        context.save();
-        context.set_state(Gtk.StateFlags.LINK);
-        let linkColor = _getColor(context);
-        this._activeNickColor = _getColor(context);
+        const context = this.get_style_context();
+        const [, activeColor] =
+            context.lookup_color('active_nick_color');
+        const [, activeHoverColor] =
+            context.lookup_color('active_nick_hover_color');
+        const [, inactiveColor] =
+            context.lookup_color('inactive_nick_color');
+        const [, inactiveHoverColor] =
+            context.lookup_color('inactive_nick_hover_color');
 
-        context.set_state(Gtk.StateFlags.LINK | Gtk.StateFlags.PRELIGHT);
-        this._hoveredLinkColor = _getColor(context);
-        context.restore();
-
-        let desaturatedNickColor = (
-            this._activeNickColor.red +
-            this._activeNickColor.blue +
-            this._activeNickColor.green) / 3;
-        this._inactiveNickColor = new Gdk.RGBA({
-            red: desaturatedNickColor,
-            green: desaturatedNickColor,
-            blue: desaturatedNickColor,
-            alpha: 1.0,
-        });
-        if (this._activeNickColor.equal(this._inactiveNickColor))
-            this._inactiveNickColor.alpha = 0.5;
-
-        context.save();
-        context.add_class('view');
-        context.set_state(Gtk.StateFlags.NORMAL);
-        this._statusHeaderHoverColor = _getColor(context);
-        context.restore();
+        this._activeNickColor = activeColor;
+        this._inactiveNickColor = inactiveColor;
+        this._hoveredLinkColor = activeHoverColor;
+        this._statusHeaderHoverColor = inactiveHoverColor;
 
         let buffer = this._view.get_buffer();
         let tagTable = buffer.get_tag_table();
         let tags = [{
             name: 'status',
-            foreground_rgba: dimColor,
+            foreground_rgba: inactiveColor,
         }, {
             name: 'timestamp',
-            foreground_rgba: dimColor,
+            foreground_rgba: inactiveColor,
         }, {
             name: 'url',
-            foreground_rgba: linkColor,
+            foreground_rgba: activeColor,
         }];
         tags.forEach(tagProps => {
             let tag = tagTable.lookup(tagProps.name);
@@ -661,7 +707,7 @@ const ChatView = GObject.registerClass({
 
     // eslint-disable-next-line camelcase
     get max_nick_chars() {
-        return this._maxNickChars;
+        return this._view.indent_width_chars;
     }
 
     // eslint-disable-next-line camelcase
@@ -670,29 +716,11 @@ const ChatView = GObject.registerClass({
     }
 
     _updateMaxNickChars(length) {
-        if (length <= this._maxNickChars)
+        if (length <= this._view.indent_width_chars)
             return;
 
-        this._maxNickChars = length;
+        this._view.indent_width_chars = length;
         this.notify('max-nick-chars');
-        this._updateIndent();
-    }
-
-    _updateIndent() {
-        let context = this._view.get_pango_context();
-        let metrics = context.get_metrics(null, null);
-        let charWidth = Math.max(
-            metrics.get_approximate_char_width(),
-            metrics.get_approximate_digit_width());
-        let pixelWidth = Pango.units_to_double(charWidth);
-
-        let totalWidth = this._maxNickChars * pixelWidth + NICK_SPACING;
-
-        let tabs = Pango.TabArray.new(1, true);
-        tabs.set_tab(0, Pango.TabAlign.LEFT, totalWidth);
-        this._view.tabs = tabs;
-        this._view.indent = -totalWidth;
-        this._view.left_margin = MARGIN + totalWidth;
     }
 
     _updateScroll() {
@@ -709,13 +737,8 @@ const ChatView = GObject.registerClass({
         }
     }
 
-    _onScroll(w, event) {
-        let [hasDir, dir] = event.get_scroll_direction();
-        if (hasDir && dir !== Gdk.ScrollDirection.UP)
-            return Gdk.EVENT_PROPAGATE;
-
-        let [hasDeltas, dx_, dy] = event.get_scroll_deltas();
-        if (hasDeltas && dy >= 0)
+    _onScroll(w, dx, dy) {
+        if (dy >= 0)
             return Gdk.EVENT_PROPAGATE;
 
         this._autoscroll = false;
@@ -723,9 +746,7 @@ const ChatView = GObject.registerClass({
         return this._fetchBacklog();
     }
 
-    _onKeyPress(w, event) {
-        let [, keyval] = event.get_keyval();
-
+    _onKeyPressed(c, keyval) {
         if (keyval === Gdk.KEY_Home ||
             keyval === Gdk.KEY_KP_Home) {
             this._view.emit('move-cursor',
@@ -808,37 +829,40 @@ const ChatView = GObject.registerClass({
         this._pending.delete(id);
     }
 
-    _showUrlContextMenu(url) {
-        let menu = new Gtk.Menu();
+    _showUrlContextMenu(url, x, y) {
+        const section = new Gio.Menu();
 
-        let item = new Gtk.MenuItem({ label: _('Open Link') });
-        item.connect('activate', () => {
-            Utils.openURL(url, Gtk.get_current_event_time());
+        section.append(
+            _('Open Link'), `chatlog.open-url("${url}")`);
+        section.append(
+            _('Copy Link Address'), `chatlog.copy-url("${url}")`);
+
+        const menu = new Gtk.PopoverMenu({
+            position: Gtk.PositionType.BOTTOM,
+            pointing_to: new Gdk.Rectangle({ x, y }),
+            menu_model: section,
+            has_arrow: false,
         });
-        menu.append(item);
-
-        item = new Gtk.MenuItem({ label: _('Copy Link Address') });
-        item.connect('activate', () => {
-            let clipboard = Gtk.Clipboard.get_default(item.get_display());
-            clipboard.set_text(url, -1);
-        });
-        menu.append(item);
-
-        menu.show_all();
-        menu.popup_at_pointer(null);
+        menu.set_parent(this._view);
+        menu.popup();
     }
 
-    _handleButtonTagsHover(view, event) {
-        let [, eventX, eventY] = event.get_coords();
-        let [x, y] = view.window_to_buffer_coords(Gtk.TextWindowType.WIDGET,
-            eventX, eventY);
-        let [inside, iter] = view.get_iter_at_location(x, y);
+    _getHoveredButtonTags(coords) {
+        let inside, iter;
 
-        let hoveredButtonTags;
-        if (inside)
-            hoveredButtonTags = iter.get_tags().filter(t => t instanceof ButtonTag);
-        else
-            hoveredButtonTags = [];
+        if (coords.length > 0) {
+            const [eventX, eventY] = coords;
+            const [x, y] = this._view.window_to_buffer_coords(
+                Gtk.TextWindowType.WIDGET, eventX, eventY);
+            [inside, iter] = this._view.get_iter_at_location(x, y);
+        }
+
+        return inside
+            ? iter.get_tags().filter(t => t instanceof ButtonTag) : [];
+    }
+
+    _handleButtonTagsHover(controller, ...coords) {
+        const hoveredButtonTags = this._getHoveredButtonTags(coords);
 
         hoveredButtonTags.forEach(t => (t.hover = true));
         this._hoveredButtonTags.forEach(t => {
@@ -848,22 +872,43 @@ const ChatView = GObject.registerClass({
         let isHovering = hoveredButtonTags.length > 0;
         let wasHovering = this._hoveredButtonTags.length > 0;
 
-        if (isHovering !== wasHovering) {
-            let cursor = isHovering ? this._hoverCursor : null;
-            this._view.get_window(Gtk.TextWindowType.TEXT).set_cursor(cursor);
-        }
+        if (isHovering !== wasHovering)
+            this._view.set_cursor_from_name(isHovering ? 'pointer' : 'text');
 
         this._hoveredButtonTags = hoveredButtonTags;
+    }
 
-        return Gdk.EVENT_PROPAGATE;
+    _handleButtonTagPressed(gesture, nPress, ...coords) {
+        const event = gesture.get_current_event();
+
+        if (nPress > 1 || !event.triggers_context_menu())
+            return;
+
+        const hoveredButtonTags = this._getHoveredButtonTags(coords);
+        hoveredButtonTags.forEach(t => t.popupMenu(...coords));
+
+        if (hoveredButtonTags.length > 0)
+            gesture.set_state(Gtk.EventSequenceState.CLAIMED);
+    }
+
+    _handleButtonTagReleased(gesture, nPress, ...coords) {
+        const button = gesture.get_current_button();
+
+        if (nPress > 1 || button !== Gdk.BUTTON_PRIMARY)
+            return;
+
+        const hoveredButtonTags = this._getHoveredButtonTags(coords);
+        hoveredButtonTags.forEach(t => t.clicked(...coords));
+
+        if (hoveredButtonTags.length > 0)
+            gesture.set_state(Gtk.EventSequenceState.CLAIMED);
     }
 
     _showLoadingIndicator() {
         let indicator = new Gtk.Image({
             icon_name: 'content-loading-symbolic',
-            visible: true,
         });
-        indicator.get_style_context().add_class('dim-label');
+        indicator.add_css_class('dim-label');
 
         let { buffer } = this._view;
         let iter = buffer.get_start_iter();
@@ -989,12 +1034,12 @@ const ChatView = GObject.registerClass({
     }
 
     _onMemberRenamed(room, oldMember, newMember) {
-        let text = _('%s is now known as %s').format(oldMember.alias, newMember.alias);
+        let text = vprintf(_('%s is now known as %s'), oldMember.alias, newMember.alias);
         this._insertStatus(text, oldMember.alias, 'renamed');
     }
 
     _onMemberDisconnected(room, member, message) {
-        let text = _('%s has disconnected').format(member.alias);
+        let text = vprintf(_('%s has disconnected'), member.alias);
         if (message)
             text += ` (${message})`;
         this._insertStatus(text, member.alias, 'left');
@@ -1003,26 +1048,26 @@ const ChatView = GObject.registerClass({
     _onMemberKicked(room, member, actor) {
         let [kicked, kicker] = [member.alias, actor ? actor.alias : null];
         let msg = kicker
-            ? _('%s has been kicked by %s').format(kicked, kicker)
-            : _('%s has been kicked').format(kicked);
+            ? vprintf(_('%s has been kicked by %s'), kicked, kicker)
+            : vprintf(_('%s has been kicked'), kicked);
         this._insertStatus(msg, kicked, 'left');
     }
 
     _onMemberBanned(room, member, actor) {
         let [banned, banner] = [member.alias, actor ? actor.alias : null];
         let msg = banner
-            ? _('%s has been banned by %s').format(banned, banner)
-            : _('%s has been banned').format(banned);
+            ? vprintf(_('%s has been banned by %s'), banned, banner)
+            : vprintf(_('%s has been banned'), banned);
         this._insertStatus(msg, banned, 'left');
     }
 
     _onMemberJoined(room, member) {
-        let text = _('%s joined').format(member.alias);
+        let text = vprintf(_('%s joined'), member.alias);
         this._insertStatus(text, member.alias, 'joined');
     }
 
     _onMemberLeft(room, member, message) {
-        let text = _('%s left').format(member.alias);
+        let text = vprintf(_('%s left'), member.alias);
 
         if (message)
             text += ` (${message})`;
@@ -1117,25 +1162,25 @@ const ChatView = GObject.registerClass({
 
         let stats = [];
         if (this._statusCount.joined > 0) {
-            stats.push(
+            stats.push(vprintf(
                 ngettext(
                     '%d user joined',
-                    '%d users joined', this._statusCount.joined)
-                .format(this._statusCount.joined));
+                    '%d users joined', this._statusCount.joined),
+                this._statusCount.joined));
         }
         if (this._statusCount.left > 0) {
-            stats.push(
+            stats.push(vprintf(
                 ngettext(
                     '%d user left',
-                    '%d users left', this._statusCount.left)
-                .format(this._statusCount.left));
+                    '%d users left', this._statusCount.left),
+                this._statusCount.left));
         }
         // TODO: How do we update the arrow direction when text direction change?
         let iter = buffer.get_iter_at_mark(headerMark);
-        let tags = [this._lookupTag('status'), headerTag];
+        let tags = [this._lookupTag('gap'), this._lookupTag('status'), headerTag];
         let headerText = stats.join(', ');
         let baseDir = Pango.find_base_dir(headerText, -1);
-        this._insertWithTags(iter, headerText, tags);
+        this._insertWithTags(iter, `${headerText}\u00A0`, tags);
         this._insertWithTags(iter,
             baseDir === Pango.Direction.LTR ? '\u25B6' : '\u25C0',
             tags.concat(headerArrowTag));
@@ -1362,7 +1407,7 @@ const ChatView = GObject.registerClass({
                 url.name, tags.concat(this._lookupTag('url'), tag));
 
             if (GLib.uri_parse_scheme(url.url).startsWith('http'))
-                previews.push(new URLPreview({ uri: url.url, visible: true }));
+                previews.push(new URLPreview({ uri: url.url }));
 
             pos = url.pos + url.name.length;
         }
@@ -1411,13 +1456,11 @@ const ChatView = GObject.registerClass({
             tag.foreground_rgba = this._inactiveNickColor;
     }
 
-    _onNickTagClicked(tag) {
-        let view = this._view;
-        let event = Gtk.get_current_event();
-        let [, eventX, eventY] = event.get_coords();
-        let [x, y] = view.window_to_buffer_coords(Gtk.TextWindowType.WIDGET,
+    _onNickTagClicked(tag, eventX, eventY) {
+        const view = this._view;
+        const [x, y] = view.window_to_buffer_coords(Gtk.TextWindowType.WIDGET,
             eventX, eventY);
-        let [inside_, start] = view.get_iter_at_location(x, y);
+        const [inside_, start] = view.get_iter_at_location(x, y);
         let end = start.copy();
 
         if (!start.starts_tag(tag))
@@ -1439,16 +1482,16 @@ const ChatView = GObject.registerClass({
 
         if (!tag._popover) {
             tag._popover = new UserPopover({
-                relative_to: this._view,
                 userTracker: this._userTracker,
                 room: this._room,
             });
+            tag._popover.set_parent(this._view);
         }
 
         tag._popover.nickname = actualNickName;
 
         tag._popover.pointing_to = rect1;
-        tag._popover.show();
+        tag._popover.popup();
     }
 
     _createUrlTag(url) {
@@ -1457,10 +1500,11 @@ const ChatView = GObject.registerClass({
             tag.foreground_rgba = tag.hover ? this._hoveredLinkColor : null;
         });
         tag.connect('clicked', () => {
-            Utils.openURL(url, Gtk.get_current_event_time());
+            const v = new GLib.Variant('s', url);
+            this._actionGroup.activate_action('open-url', v);
         });
-        tag.connect('popup-menu', () => {
-            this._showUrlContextMenu(url);
+        tag.connect('popup-menu', (t, ...coords) => {
+            this._showUrlContextMenu(url, ...coords);
         });
         return tag;
     }
