@@ -1,17 +1,11 @@
-export {
-    ConnectionProperties,
-    ConnectionDetails,
-    ConnectionsList
-};
-
 import Gio from 'gi://Gio';
 import GLib from 'gi://GLib';
 import GObject from 'gi://GObject';
 import Gtk from 'gi://Gtk';
 import Tp from 'gi://TelepathyGLib';
 
-import { AccountsMonitor } from './accountsMonitor.js';
-import { NetworksManager } from './networksManager.js';
+import AccountsMonitor from './accountsMonitor.js';
+import NetworksManager from './networksManager.js';
 import * as Utils from './utils.js';
 
 Gio._promisify(Tp.Account.prototype,
@@ -55,19 +49,17 @@ class ConnectionRow extends Gtk.ListBoxRow {
             margin_top: 12,
             margin_bottom: 12,
         });
-        this.add(box);
+        this.set_child(box);
 
-        box.add(new Gtk.Label({ label: name, halign: Gtk.Align.START }));
+        box.append(new Gtk.Label({ label: name, halign: Gtk.Align.START }));
 
         let insensitiveDesc = new Gtk.Label({
             label: _('Already added'),
             hexpand: true,
-            no_show_all: true,
             halign: Gtk.Align.END,
+            visible: false,
         });
-        box.add(insensitiveDesc);
-
-        this.show_all();
+        box.append(insensitiveDesc);
 
         this.bind_property('sensitive',
             insensitiveDesc, 'visible',
@@ -80,7 +72,7 @@ class ConnectionRow extends Gtk.ListBoxRow {
     }
 });
 
-const ConnectionsList = GObject.registerClass({
+export const ConnectionsList = GObject.registerClass({
     Properties: {
         'favorites-only': GObject.ParamSpec.boolean(
             'favorites-only', 'favorites-only', 'favorites-only',
@@ -99,9 +91,9 @@ const ConnectionsList = GObject.registerClass({
 
         this.hscrollbar_policy = Gtk.PolicyType.NEVER;
 
-        this._list = new Gtk.ListBox({ visible: true });
+        this._list = new Gtk.ListBox();
         this._list.connect('row-activated', this._onRowActivated.bind(this));
-        this.add(this._list);
+        this.set_child(this._list);
 
         this._rows = new Map();
 
@@ -114,19 +106,16 @@ const ConnectionsList = GObject.registerClass({
             halign: Gtk.Align.CENTER,
             valign: Gtk.Align.CENTER,
             orientation: Gtk.Orientation.VERTICAL,
-            visible: true,
         });
-        placeholder.add(new Gtk.Image({
+        placeholder.append(new Gtk.Image({
             icon_name: 'edit-find-symbolic',
             pixel_size: 115,
-            visible: true,
         }));
-        placeholder.add(new Gtk.Label({
+        placeholder.append(new Gtk.Label({
             label: _('No results.'),
-            visible: true,
         }));
 
-        placeholder.get_style_context().add_class('dim-label');
+        placeholder.add_css_class('dim-label');
 
         this._list.set_placeholder(placeholder);
 
@@ -198,7 +187,10 @@ const ConnectionsList = GObject.registerClass({
     }
 
     _networksChanged() {
-        this._list.foreach(w => w.destroy());
+        [...this._list].forEach(w => {
+            this._list.remove(w);
+            w.run_dispose();
+        });
 
         let { accounts } = this._accountsMonitor;
         let usedNetworks = accounts.filter(a => a.predefined).map(a => a.service);
@@ -213,7 +205,7 @@ const ConnectionsList = GObject.registerClass({
                 id: network.id,
                 sensitive,
             }));
-            this._list.add(this._rows.get(network.id));
+            this._list.append(this._rows.get(network.id));
         });
     }
 
@@ -267,7 +259,7 @@ const ConnectionsList = GObject.registerClass({
     }
 });
 
-const ConnectionDetails = GObject.registerClass({
+export const ConnectionDetails = GObject.registerClass({
     Template: 'resource:///org/gnome/Polari/ui/connection-details.ui',
     InternalChildren: [
         'nameEntry',
@@ -317,7 +309,7 @@ const ConnectionDetails = GObject.registerClass({
 
         let realnameStore = new Gtk.ListStore();
         realnameStore.set_column_types([GObject.TYPE_STRING]);
-        realnameStore.insert_with_valuesv(-1, [0], [GLib.get_real_name()]);
+        realnameStore.insert_with_values(-1, [0], [GLib.get_real_name()]);
 
         let completion = new Gtk.EntryCompletion({
             model: realnameStore,
@@ -332,14 +324,14 @@ const ConnectionDetails = GObject.registerClass({
 
     setErrorHint(hint) {
         if (hint === ErrorHint.SERVER)
-            this._serverEntry.get_style_context().add_class('error');
+            this._serverEntry.add_css_class('error');
         else
-            this._serverEntry.get_style_context().remove_class('error');
+            this._serverEntry.remove_css_class('error');
 
         if (hint === ErrorHint.NICK)
-            this._nickEntry.get_style_context().add_class('error');
+            this._nickEntry.add_css_class('error');
         else
-            this._nickEntry.get_style_context().remove_class('error');
+            this._nickEntry.remove_css_class('error');
     }
 
     _getParams() {
@@ -508,7 +500,7 @@ const ConnectionDetails = GObject.registerClass({
 });
 
 
-const ConnectionProperties = GObject.registerClass({
+export const ConnectionProperties = GObject.registerClass({
     Template: 'resource:///org/gnome/Polari/ui/connection-properties.ui',
     InternalChildren: [
         'details',
@@ -519,7 +511,7 @@ const ConnectionProperties = GObject.registerClass({
     _init(account) {
         /* Translators: %s is a connection name */
         super._init({
-            title: _('“%s” Properties').format(account.display_name),
+            title: vprintf(_('“%s” Properties'), account.display_name),
             use_header_bar: 1,
         });
 
