@@ -1,22 +1,15 @@
-export {
-    ChatEntry,
-    EntryArea,
-    NickPopover
-};
-
 import Gdk from 'gi://Gdk';
 import GdkPixbuf from 'gi://GdkPixbuf';
 import Gio from 'gi://Gio';
 import GLib from 'gi://GLib';
 import GObject from 'gi://GObject';
-import Gspell from 'gi://Gspell';
 import Gtk from 'gi://Gtk';
 import Tp from 'gi://TelepathyGLib';
 
 import { MAX_NICK_CHARS } from './chatView.js';
-import { DropTargetIface } from './pasteManager.js';
-import { IrcParser } from './ircParser.js';
-import { TabCompletion } from './tabCompletion.js';
+import { DropTargetIface, gtypeFromFormats } from './pasteManager.js';
+import IrcParser from './ircParser.js';
+import TabCompletion from './tabCompletion.js';
 
 const MAX_NICK_UPDATE_TIME = 5; /* s */
 const MAX_LINES = 5;
@@ -24,7 +17,7 @@ const MAX_LINES = 5;
 Gio._promisify(Gio._LocalFilePrototype,
     'query_info_async', 'query_info_finish');
 
-const ChatEntry = GObject.registerClass({
+export const ChatEntry = GObject.registerClass({
     Implements: [DropTargetIface],
     Properties: {
         'can-drop': GObject.ParamSpec.override('can-drop', DropTargetIface),
@@ -35,31 +28,21 @@ const ChatEntry = GObject.registerClass({
         'file-pasted': { param_types: [Gio.File.$gtype] },
     },
 }, class ChatEntry extends Gtk.Entry {
-    static get _checker() {
-        if (!this.__checker)
-            this.__checker = new Gspell.Checker();
-        return this.__checker;
-    }
-
     _init(params) {
         super._init(params);
 
-        DropTargetIface.addTargets(this, this);
+        const delegate = this.get_delegate();
+        this.addTargets(delegate);
 
         let app = Gio.Application.get_default();
         let action = app.lookup_action('show-emoji-picker');
         action.connect('activate', () => {
             if (this.is_sensitive() && this.get_mapped())
-                this.emit('insert-emoji');
+                delegate.emit('insert-emoji');
         });
 
-        let buffer = Gspell.EntryBuffer.get_from_gtk_entry_buffer(this.buffer);
-        buffer.set_spell_checker(ChatEntry._checker);
-
-        let spellEntry = Gspell.Entry.get_from_gtk_entry(this);
-        spellEntry.set_inline_spell_checking(true);
-
-        this._useDefaultHandler = false;
+        delegate.connect('insert-text', this._onInsertText.bind(this));
+        delegate.connect('paste-clipboard', this._onPasteClipboard.bind(this));
     }
 
     // eslint-disable-next-line camelcase
@@ -67,55 +50,49 @@ const ChatEntry = GObject.registerClass({
         return true;
     }
 
-    vfunc_drag_data_received(context, x, y, data, info, time) {
-        let str = data.get_text();
-        if (!str || str.split('\n').length >= MAX_LINES)
-            // Disable GtkEntry's built-in drop target support
+    _onInsertText(editable, text) {
+        const nLines = text.split('\n').length;
+        if (nLines < MAX_LINES)
             return;
 
-        GObject.signal_stop_emission_by_name(this, 'drag-data-received');
-        super.vfunc_drag_data_received(context, x, y, data, info, time);
+        editable.stop_emission_by_name('insert-text');
+        this.emit('text-pasted', text, nLines);
     }
 
-    vfunc_paste_clipboard() {
-        if (!this.editable || this._useDefaultHandler) {
-            super.vfunc_paste_clipboard();
+    async _onPasteClipboard(editable) {
+        if (!this.editable)
             return;
-        }
 
-        let clipboard = Gtk.Clipboard.get_default(this.get_display());
-        clipboard.request_uris((cb, uris) => {
-            if (uris && uris.length)
-                this.emit('file-pasted', Gio.File.new_for_uri(uris[0]));
-            else
-                clipboard.request_text(this._onTextReceived.bind(this));
-        });
+        editable.stop_emission_by_name('paste-clipboard');
+        const clipboard = this.get_clipboard();
 
-        clipboard.request_image((cb, pixbuf) => {
-            if (!pixbuf)
-                return;
-            this.emit('image-pasted', pixbuf);
-        });
+        const { formats } = clipboard;
+        const type = gtypeFromFormats(formats);
+        const value = await this._readClipboardValue(clipboard, type);
+        if (typeof value === 'string')
+            editable.emit('insert-at-cursor', value);
+        else if (value instanceof GdkPixbuf.Pixbuf)
+            this.emit('image-pasted', value);
+        else if (value instanceof Gio.File)
+            this.emit('file-pasted', value);
     }
 
-    _onTextReceived(clipboard, text) {
-        if (!text)
-            return;
-        text = text.trim();
-
-        let nLines = text.split('\n').length;
-        if (nLines >= MAX_LINES) {
-            this.emit('text-pasted', text, nLines);
-            return;
-        }
-
-        this._useDefaultHandler = true;
-        this.emit('paste-clipboard');
-        this._useDefaultHandler = false;
+    _readClipboardValue(clipboard, type) {
+        return new Promise((resolve, reject) => {
+            clipboard.read_value_async(type, 0, null, (_, res) => {
+                try {
+                    const value = clipboard.read_value_finish(res);
+                    resolve(value);
+                } catch (e) {
+                    console.error(e);
+                    reject(e);
+                }
+            });
+        });
     }
 });
 
-const NickPopover = GObject.registerClass({
+export const NickPopover = GObject.registerClass({
     Template: 'resource:///org/gnome/Polari/ui/nick-popover.ui',
     InternalChildren: [
         'nickEntry',
@@ -163,7 +140,7 @@ const NickPopover = GObject.registerClass({
     }
 });
 
-const EntryArea = GObject.registerClass({
+export default GObject.registerClass({
     Template: 'resource:///org/gnome/Polari/ui/entry-area.ui',
     InternalChildren: [
         'chatEntry',
@@ -196,22 +173,29 @@ const EntryArea = GObject.registerClass({
         this._ircParser = new IrcParser(this._room);
         this._maxNickChars = MAX_NICK_CHARS;
         this._nickChangedId = 0;
+        this._popoverClosedId = 0;
 
         super._init(params);
 
         this.connect('destroy', this._onDestroy.bind(this));
         this.connect('notify::sensitive', this._onSensitiveChanged.bind(this));
         this.connect('realize', () => {
-            this._toplevel = this.get_toplevel();
-            this._keyPressId = this._toplevel.connect('key-press-event',
-                this._onKeyPressEvent.bind(this));
+            this._toplevel = this.get_root();
+            this._toplevelKeyController = new Gtk.EventControllerKey();
+            this._toplevelKeyController.connect('key-pressed',
+                this._onKeyPressed.bind(this));
+            this._toplevel.add_controller(this._toplevelKeyController);
         });
         this.connect('map', () => {
-            this._nickButton.popover = EntryArea._nickPopover;
+            EntryArea._nickPopover.unparent();
+            EntryArea._nickPopover.set_parent(this._nickButton);
 
             if (this._nickChangedId)
                 return;
 
+            this._popoverCloseId = EntryArea._nickPopover.connect('closed',
+                () => (this._nickButton.active = false));
+
             this._nickChangedId = EntryArea._nickPopover.connect('nick-changed',
                 () => {
                     this._setNick(EntryArea._nickPopover.nick);
@@ -223,22 +207,20 @@ const EntryArea = GObject.registerClass({
             if (this._nickChangedId)
                 EntryArea._nickPopover.disconnect(this._nickChangedId);
             this._nickChangedId = 0;
+
+            if (this._popoverClosedId)
+                EntryArea._nickPopover.disconnect(this._popoverClosedId);
+            this._popoverClosedId = 0;
+        });
+        this._nickButton.connect('toggled', () => {
+            if (this._nickButton.active)
+                EntryArea._nickPopover.popup();
+            else
+                EntryArea._nickPopover.popdown();
         });
 
-        this._nickLabel.set_state_flags(Gtk.StateFlags.LINK, false);
         this._nickLabel.width_chars = this._maxNickChars;
 
-        /* HACK: We don't want the button to look different when the toplevel
-                 is unfocused, so filter out the BACKDROP state */
-        this._nickButton.connect('state-flags-changed', w => {
-            let state = w.get_state_flags();
-            if (!(state & Gtk.StateFlags.BACKDROP))
-                return; // avoid indefinite recursion
-
-            state &= ~Gtk.StateFlags.BACKDROP;
-            w.set_state_flags(state, true);
-        });
-
         this._chatEntry.connect('text-pasted', (entry, text, nLines) => {
             this.pasteText(text, nLines);
         });
@@ -266,7 +248,7 @@ const EntryArea = GObject.registerClass({
             if (this._ircParser.process(this._chatEntry.text)) {
                 this._chatEntry.text = '';
             } else {
-                this._chatEntry.get_style_context().add_class('error');
+                this._chatEntry.add_css_class('error');
                 this._chatEntry.grab_focus(); // select text
             }
         });
@@ -274,18 +256,18 @@ const EntryArea = GObject.registerClass({
         this._cancelButton.connect('clicked', this._onCancelClicked.bind(this));
         this._pasteButton.connect('clicked', this._onPasteClicked.bind(this));
 
-        this._pasteBox.connect_after('key-press-event', (w, event) => {
-            let [, keyval] = event.get_keyval();
-            let [, mods] = event.get_state();
+        this._pasteController = new Gtk.EventControllerKey();
+        this._pasteController.connect_after('key-pressed', (c, keyval, code, mods) => {
             if (keyval === Gdk.KEY_Escape ||
                 keyval === Gdk.KEY_BackSpace ||
                 keyval === Gdk.KEY_Delete ||
                 keyval === Gdk.KEY_z && mods & Gdk.ModifierType.CONTROL_MASK) {
-                this._cancelButton.clicked();
+                this._cancelButton.emit('clicked');
                 return Gdk.EVENT_STOP;
             }
             return Gdk.EVENT_PROPAGATE;
         });
+        this._pasteBox.add_controller(this._pasteController);
 
         if (!this._room)
             return;
@@ -326,22 +308,20 @@ const EntryArea = GObject.registerClass({
     }
 
     _canFocusChatEntry() {
-        let toplevelFocus = this._chatEntry.get_toplevel().get_focus();
+        const toplevelFocus = this._chatEntry.get_root().get_focus();
         return this.sensitive &&
                this._chatEntry.get_mapped() &&
                !this._chatEntry['has-focus'] &&
                !(toplevelFocus instanceof Gtk.Entry);
     }
 
-    _onKeyPressEvent(w, event) {
+    _onKeyPressed(controller, keyval, keycode, state) {
         if (!this._canFocusChatEntry())
             return Gdk.EVENT_PROPAGATE;
 
-        let [, keyval] = event.get_keyval();
         if (Gdk.keyval_to_unicode(keyval) === 0)
             return Gdk.EVENT_PROPAGATE;
 
-        let [, state] = event.get_state();
         if (state !== 0 && state !== Gdk.ModifierType.SHIFT_MASK)
             return Gdk.EVENT_PROPAGATE;
 
@@ -355,12 +335,12 @@ const EntryArea = GObject.registerClass({
             return Gdk.EVENT_PROPAGATE;
 
         this._chatEntry.grab_focus_without_selecting();
-        this._chatEntry.event(event);
+        controller.forward(this._chatEntry.get_delegate());
         return Gdk.EVENT_STOP;
     }
 
     _onEntryChanged() {
-        this._chatEntry.get_style_context().remove_class('error');
+        this._chatEntry.remove_css_class('error');
     }
 
     _setPasteContent(content) {
@@ -377,14 +357,16 @@ const EntryArea = GObject.registerClass({
     }
 
     pasteText(text, nLines) {
-        this._confirmLabel.label = ngettext(
-            'Paste %s line of text to public paste service?',
-            'Paste %s lines of text to public paste service?', nLines)
-        .format(nLines);
-        this._uploadLabel.label = ngettext(
-            'Uploading %s line of text to public paste service…',
-            'Uploading %s lines of text to public paste service…', nLines)
-        .format(nLines);
+        this._confirmLabel.label = vprintf(
+            ngettext(
+                'Paste %s line of text to public paste service?',
+                'Paste %s lines of text to public paste service?', nLines),
+            nLines);
+        this._uploadLabel.label = vprintf(
+            ngettext(
+                'Uploading %s line of text to public paste service…',
+                'Uploading %s lines of text to public paste service…', nLines),
+            nLines);
         this._setPasteContent(text);
     }
 
@@ -407,9 +389,9 @@ const EntryArea = GObject.registerClass({
 
         let name = fileInfo.get_display_name();
         /* Translators: %s is a filename */
-        this._confirmLabel.label = _('Upload “%s” to public paste service?').format(name);
+        this._confirmLabel.label = vprintf(_('Upload “%s” to public paste service?'), name);
         /* Translators: %s is a filename */
-        this._uploadLabel.label = _('Uploading “%s” to public paste service…').format(name);
+        this._uploadLabel.label = vprintf(_('Uploading “%s” to public paste service…'), name);
         this._setPasteContent(file);
     }
 
@@ -418,9 +400,9 @@ const EntryArea = GObject.registerClass({
         let nick = this._room.channel.connection.self_contact.alias;
         if (this._room.type === Tp.HandleType.ROOM)
             /* translators: %s is a nick, #%s a channel */
-            title = _('%s in #%s').format(nick, this._room.display_name);
+            title = vprintf(_('%s in #%s'), nick, this._room.display_name);
         else
-            title = _('Paste from %s').format(nick);
+            title = vprintf(_('Paste from %s'), nick);
         this._confirmLabel.hide();
 
         this._confirmLabel.hide();
@@ -431,12 +413,13 @@ const EntryArea = GObject.registerClass({
             const url =
                 await app.pasteManager.pasteContent(this._pasteContent, title);
             this._setPasteContent(null);
-            this._chatEntry.emit('insert-at-cursor', url);
+            this._chatEntry.get_delegate().emit('insert-at-cursor', url);
         } catch (e) {
             let type = typeof this._pasteContent;
             if (type === 'object')
                 type = this._pasteContent.toString();
-            debug(`Failed to paste content of type ${type}`);
+            console.warn(`Failed to paste content of type ${type}`);
+            console.debug(e);
         } finally {
             this._uploadSpinner.stop();
         }
@@ -449,6 +432,11 @@ const EntryArea = GObject.registerClass({
     _onSensitiveChanged() {
         if (this._canFocusChatEntry())
             this._chatEntry.grab_focus();
+
+        if (this.sensitive)
+            this._nickLabel.add_css_class('polari-active-nick');
+        else
+            this._nickLabel.remove_css_class('polari-active-nick');
     }
 
     _onChannelChanged(room) {
@@ -516,11 +504,11 @@ const EntryArea = GObject.registerClass({
         if (this._channelChangedId)
             this._room.disconnect(this._channelChangedId);
         this._channelChangedId = 0;
-        if (this._keyPressId)
-            this._toplevel.disconnect(this._keyPressId);
-        this._keyPressId = 0;
         if (this._nickChangedId)
             EntryArea._nickPopover.disconnect(this._nickChangedId);
         this._nickChangedId = 0;
+        if (this._toplevelKeyController)
+            this._toplevel.remove_controller(this._toplevelKeyController);
+        this._toplevelKeyController = null;
     }
 });
