@@ -1,5 +1,3 @@
-export { IrcParser };
-
 import Gio from 'gi://Gio';
 import GLib from 'gi://GLib';
 import Tp from 'gi://TelepathyGLib';
@@ -7,7 +5,7 @@ import Tp from 'gi://TelepathyGLib';
 const Signals = imports.signals;
 
 import * as AppNotifications from './appNotifications.js';
-import { RoomManager } from './roomManager.js';
+import RoomManager from './roomManager.js';
 import * as Utils from './utils.js';
 
 Gio._promisify(Tp.Account.prototype,
@@ -50,7 +48,7 @@ const UNKNOWN_COMMAND_MESSAGE =
 
 const ROOM_PREFIXES = ['#', '&', '+', '!'];
 
-const IrcParser = class IrcParser {
+export default class IrcParser {
     constructor(room) {
         this._app = Gio.Application.get_default();
         this._roomManager = RoomManager.getDefault();
@@ -62,7 +60,7 @@ const IrcParser = class IrcParser {
     }
 
     _createFeedbackUsage(cmd) {
-        return this._createFeedbackLabel(_('Usage: %s').format(_(knownCommands[cmd])));
+        return this._createFeedbackLabel(vprintf(_('Usage: %s'), _(knownCommands[cmd])));
     }
 
     _createFeedbackGrid(header, items) {
@@ -129,16 +127,16 @@ const IrcParser = class IrcParser {
                 break;
             }
             if (argv.length)
-                log(`Excess arguments to JOIN command: ${argv}`);
+                console.warn(`Excess arguments to JOIN command: ${argv}`);
             if (!ROOM_PREFIXES.some(prefix => room.startsWith(prefix)))
                 room = `#${room}`;
             let { account } = this._room;
             let app = Gio.Application.get_default();
             let action = app.lookup_action('join-room');
-            action.activate(GLib.Variant.new('(ssu)', [
+            action.activate(GLib.Variant.new('(ssb)', [
                 account.get_object_path(),
                 room,
-                Utils.getTpEventTime(),
+                true,
             ]));
             break;
         }
@@ -184,11 +182,11 @@ const IrcParser = class IrcParser {
 
             let app = Gio.Application.get_default();
             let action = app.lookup_action('message-user');
-            action.activate(GLib.Variant.new('(sssu)', [
+            action.activate(GLib.Variant.new('(sssb)', [
                 account.get_object_path(),
                 nick,
                 message,
-                Tp.USER_ACTION_TIME_NOT_USER_ACTION,
+                false,
             ]));
             break;
         }
@@ -196,7 +194,7 @@ const IrcParser = class IrcParser {
             let { channel } = this._room;
             let members = channel.group_dup_members_contacts().map(m => m.alias);
             output = this._createFeedbackGrid(
-                _('Users on %s:').format(channel.identifier), members);
+                vprintf(_('Users on %s:'), channel.identifier), members);
             break;
         }
         case 'NICK': {
@@ -207,7 +205,7 @@ const IrcParser = class IrcParser {
                 break;
             }
             if (argv.length)
-                log(`Excess arguments to NICK command: ${argv}`);
+                console.warn(`Excess arguments to NICK command: ${argv}`);
 
             this._app.setAccountNick(this._room.account, nick);
             break;
@@ -241,11 +239,11 @@ const IrcParser = class IrcParser {
 
             let app = Gio.Application.get_default();
             let action = app.lookup_action('message-user');
-            action.activate(GLib.Variant.new('(sssu)', [
+            action.activate(GLib.Variant.new('(sssb)', [
                 account.get_object_path(),
                 nick,
                 '',
-                Utils.getTpEventTime(),
+                true,
             ]));
             break;
         }
@@ -297,7 +295,7 @@ const IrcParser = class IrcParser {
         }
 
         if (output)
-            this._app.commandOutputQueue.addNotification(output);
+            this._app.active_window?.queueCommandOutput(output);
         return retval;
     }
 
@@ -312,7 +310,7 @@ const IrcParser = class IrcParser {
                     [last] = info[i].field_value;
             }
         }
-        return _('User: %s - Last activity: %s').format(fn ? fn : user.alias, Utils.formatTimePassed(last));
+        return vprintf(_('User: %s - Last activity: %s'), fn ? fn : user.alias, Utils.formatTimePassed(last));
     }
 
     _sendText(text) {
@@ -329,5 +327,5 @@ const IrcParser = class IrcParser {
             logError(e, 'Failed to send message');
         }
     }
-};
+}
 Signals.addSignalMethods(IrcParser.prototype);
