@@ -1,20 +1,16 @@
-export { JoinDialog };
-
-import Gdk from 'gi://Gdk';
 import Gio from 'gi://Gio';
 import GLib from 'gi://GLib';
 import GObject from 'gi://GObject';
 import Gtk from 'gi://Gtk';
 
-import { AccountsMonitor } from './accountsMonitor.js';
-import * as Utils from './utils.js';
+import AccountsMonitor from './accountsMonitor.js';
 
 const DialogPage = {
     MAIN: 0,
     CONNECTION: 1,
 };
 
-const JoinDialog = GObject.registerClass({
+export default GObject.registerClass({
     Template: 'resource:///org/gnome/Polari/ui/join-room-dialog.ui',
     InternalChildren: [
         'cancelButton',
@@ -29,28 +25,13 @@ const JoinDialog = GObject.registerClass({
         'details',
         'addButton',
         'customToggle',
+        'backButton',
     ],
 }, class JoinDialog extends Gtk.Dialog {
     _init(params) {
         params['use-header-bar'] = 1;
         super._init(params);
 
-        // TODO: Is there really no way to do this in the template?
-        let icon = new Gtk.Image({ icon_name: 'go-previous-symbolic' });
-        this._backButton = new Gtk.Button({
-            image: icon,
-            valign: Gtk.Align.CENTER,
-            focus_on_click: false,
-        });
-        this.get_header_bar().pack_start(this._backButton);
-
-        let accelGroup = new Gtk.AccelGroup();
-        this._connectionButton.add_accelerator('clicked',
-            accelGroup, Gdk.KEY_n, Gdk.ModifierType.CONTROL_MASK, 0);
-        this._backButton.add_accelerator('clicked',
-            accelGroup, Gdk.KEY_Left, Gdk.ModifierType.MOD1_MASK, 0);
-        this.add_accel_group(accelGroup);
-
         this._setupMainPage();
         this._setupConnectionPage();
 
@@ -129,7 +110,7 @@ const JoinDialog = GObject.registerClass({
             let childName = isCustom ? 'custom' : 'predefined';
             this._connectionStack.visible_child_name = childName;
             if (isCustom) {
-                this._addButton.grab_default();
+                this.set_default_widget(this._addButton);
                 this._details.reset();
             }
         });
@@ -175,10 +156,10 @@ const JoinDialog = GObject.registerClass({
 
             let app = Gio.Application.get_default();
             let action = app.lookup_action('join-room');
-            action.activate(GLib.Variant.new('(ssu)', [
+            action.activate(GLib.Variant.new('(ssb)', [
                 account.get_object_path(),
                 room,
-                Utils.getTpEventTime(),
+                true,
             ]));
         });
     }
