@@ -1,12 +1,3 @@
-export {
-    MainWindow,
-    FixedSizeFrame,
-    RoomList,
-    RoomStack,
-    UserList
-};
-
-import Gdk from 'gi://Gdk';
 import Gio from 'gi://Gio';
 import GLib from 'gi://GLib';
 import GObject from 'gi://GObject';
@@ -14,15 +5,16 @@ import Gtk from 'gi://Gtk';
 import Polari from 'gi://Polari';
 import Tp from 'gi://TelepathyGLib';
 
-import { AccountsMonitor } from './accountsMonitor.js';
-import { JoinDialog } from './joinDialog.js';
-import * as RoomList from './roomList.js'; // used in template
-import { RoomManager } from './roomManager.js';
-import * as RoomStack from './roomStack.js'; // used in template
-import * as UserList from './userList.js'; // used in template
+import AccountsMonitor from './accountsMonitor.js';
+import * as AppNotifications from './appNotifications.js';
+import JoinDialog from './joinDialog.js';
+import RoomList_ from './roomList.js'; // used in template
+import RoomManager from './roomManager.js';
+import RoomStack_ from './roomStack.js'; // used in template
+import * as UserList_ from './userList.js'; // used in template
 import * as Utils from './utils.js';
 
-const FixedSizeFrame = GObject.registerClass({
+export const FixedSizeFrame = GObject.registerClass({
     Properties: {
         height: GObject.ParamSpec.int(
             'height', 'height', 'height',
@@ -33,16 +25,19 @@ const FixedSizeFrame = GObject.registerClass({
             GObject.ParamFlags.READWRITE,
             -1, GLib.MAXINT32, -1),
     },
-}, class FixedSizeFrame extends Gtk.Bin {
+}, class FixedSizeFrame extends Gtk.Widget {
     _init(params) {
         this._height = -1;
         this._width = -1;
 
-        super._init(params);
+        super._init({
+            ...params,
+            layout_manager: new Gtk.BinLayout(),
+        });
     }
 
     _queueRedraw() {
-        let child = this.get_child();
+        const [child] = this;
         child?.queue_resize();
         this.queue_draw();
     }
@@ -74,18 +69,29 @@ const FixedSizeFrame = GObject.registerClass({
         this._queueRedraw();
     }
 
-    vfunc_get_preferred_width_for_height(forHeight) {
-        let [min, nat] = super.vfunc_get_preferred_width_for_height(forHeight);
-        return [min, this._width < 0 ? nat : this._width];
-    }
-
-    vfunc_get_preferred_height_for_width(forWidth) {
-        let [min, nat] = super.vfunc_get_preferred_height_for_width(forWidth);
-        return [min, this._height < 0 ? nat : this._height];
+    vfunc_measure(orientation, forSize) {
+        const fixedSize = orientation === Gtk.Orientation.HORIZONTAL
+            ? this._width : this._height;
+        const [min, nat] = super.vfunc_measure(orientation, forSize);
+        return [min, fixedSize < 0 ? nat : fixedSize, -1, -1];
     }
 });
 
-const MainWindow = GObject.registerClass({
+export const HeaderBarButtonLayout = GObject.registerClass(
+class ButtonBinLayout extends Gtk.BinLayout {
+    vfunc_measure(widget, orientation, forSize) {
+        const [min, nat] = super.vfunc_measure(widget, orientation, forSize);
+        if (orientation === Gtk.Orientation.VERTICAL)
+            return [min, nat, -1, -1];
+
+        let [, height] = widget.measure(Gtk.Orientation.VERTICAL, -1);
+        const padding = widget.get_style_context().get_padding();
+        height -= padding.left + padding.right;
+        return [min, Math.max(nat, height), -1, -1];
+    }
+});
+
+export default GObject.registerClass({
     Template: 'resource:///org/gnome/Polari/ui/main-window.ui',
     InternalChildren: [
         'titlebarRight',
@@ -134,6 +140,8 @@ const MainWindow = GObject.registerClass({
 
         super._init(params);
 
+        this._userListPopover.set_parent(this._showUserListButton);
+
         this._settings = new Gio.Settings({ schema_id: 'org.gnome.Polari' });
         this._gtkSettings = Gtk.Settings.get_default();
 
@@ -141,40 +149,30 @@ const MainWindow = GObject.registerClass({
         this._isMaximized = false;
         this._isFullscreen = false;
 
-        let app = this.application;
-        this._overlay.add_overlay(app.notificationQueue);
-        this._overlay.add_overlay(app.commandOutputQueue);
+        this._notificationQueue = new AppNotifications.NotificationQueue();
+        this._commandOutputQueue = new AppNotifications.CommandOutputQueue();
 
+        this._overlay.add_overlay(this._notificationQueue);
+        this._overlay.add_overlay(this._commandOutputQueue);
+
+        const app = this.application;
         if (app.isTestInstance)
-            this.get_style_context().add_class('test-instance');
+            this.add_css_class('test-instance');
         if (GLib.get_application_name().toLowerCase().includes('snapshot'))
-            this.get_style_context().add_class('snapshot');
+            this.add_css_class('snapshot');
 
-        this._viewHeight = this._calculateViewHeight();
-        this._roomStack.connect('size-allocate', () => {
-            let oldViewHeight = this._viewHeight;
-            let newViewHeight = this._calculateViewHeight();
-            if (oldViewHeight !== newViewHeight) {
-                this._viewHeight = newViewHeight;
-                this.notify('view-height');
-            }
-        });
+        this._roomStack.connect('notify::view-height',
+            () => this.notify('view-height'));
 
         // command output notifications should not pop up over
         // the input area, but appear to emerge from it, so
         // set up an appropriate margin
         this._roomStack.bind_property('entry-area-height',
-            app.commandOutputQueue, 'margin-bottom',
+            this._commandOutputQueue, 'margin-bottom',
             GObject.BindingFlags.SYNC_CREATE);
 
         // Make sure user-list button is at least as wide as icon buttons
-        this._joinButton.connect('size-allocate', (w, rect) => {
-            let { width } = rect;
-            GLib.idle_add(GLib.PRIORITY_DEFAULT_IDLE, () => {
-                this._showUserListButton.width_request = width;
-                return GLib.SOURCE_REMOVE;
-            });
-        });
+        this._showUserListButton.layout_manager = new HeaderBarButtonLayout();
 
         this._accountsMonitor = AccountsMonitor.getDefault();
         this._accountsChangedId = this._accountsMonitor.connect(
@@ -211,8 +209,14 @@ const MainWindow = GObject.registerClass({
             this._updateDecorations.bind(this));
         this._updateDecorations();
 
-        this.connect('window-state-event', this._onWindowStateEvent.bind(this));
-        this.connect('size-allocate', this._onSizeAllocate.bind(this));
+        this.connect('notify::maximized',
+            () => (this._isMaximized = this.maximized));
+        this.connect('notify::fullscreened',
+            () => (this._isFullscreen = this.fullscreened));
+        this.connect('notify::default-width',
+            () => (this._currentSize = this.get_default_size()));
+        this.connect('notify::default-height',
+            () => (this._currentSize = this.get_default_size()));
         this.connect('destroy', this._onDestroy.bind(this));
         this.connect('notify::active-room', () => {
             this._updateUserListLabel();
@@ -235,13 +239,23 @@ const MainWindow = GObject.registerClass({
         return this._subtitle.length > 0;
     }
 
-    _calculateViewHeight() {
-        return this._roomStack.get_allocated_height() - this._roomStack.entry_area_height;
-    }
-
     // eslint-disable-next-line camelcase
     get view_height() {
-        return this._viewHeight;
+        return this._roomStack.view_height;
+    }
+
+    /**
+     * @param {AppNotification} n - a notification
+     */
+    queueNotification(n) {
+        this._notificationQueue.addNotification(n);
+    }
+
+    /**
+     * @param {AppNotification} n - a notification
+     */
+    queueCommandOutput(n) {
+        this._commandOutputQueue.addNotification(n);
     }
 
     _onAccountsReachableChanged() {
@@ -250,18 +264,6 @@ const MainWindow = GObject.registerClass({
             accounts.length > 0 && !accounts.some(a => a.reachable);
     }
 
-    _onWindowStateEvent(widget, event) {
-        let state = event.get_window().get_state();
-
-        this._isFullscreen = (state & Gdk.WindowState.FULLSCREEN) !== 0;
-        this._isMaximized = (state & Gdk.WindowState.MAXIMIZED) !== 0;
-    }
-
-    _onSizeAllocate() {
-        if (!this._isFullscreen && !this._isMaximized)
-            this._currentSize = this.get_size();
-    }
-
     _onDestroy() {
         this._settings.set_boolean('window-maximized', this._isMaximized);
         this._settings.set_value('window-size',
@@ -281,15 +283,13 @@ const MainWindow = GObject.registerClass({
             this._settings.reset('last-selected-channel');
 
         this.active_room = null;
+        this._userListPopover.unparent();
 
         this._accountsMonitor.disconnect(this._accountsChangedId);
         this._accountsMonitor.disconnect(this._accountReachableId);
 
         this._roomManager.disconnect(this._roomsLoadedId);
         this._roomManager.disconnect(this._roomRemovedId);
-
-        this._overlay.remove(this.application.notificationQueue);
-        this._overlay.remove(this.application.commandOutputQueue);
     }
 
     _onAccountsChanged() {
@@ -405,10 +405,11 @@ const MainWindow = GObject.registerClass({
             this._room.channel.has_interface(Tp.IFACE_CHANNEL_INTERFACE_GROUP))
             numMembers = this._room.channel.group_dup_members_contacts().length;
 
-        let accessibleName = ngettext(
+        let accessibleName = vprintf(ngettext(
             '%d user',
-            '%d users', numMembers).format(numMembers);
-        this._showUserListButton.get_accessible().set_name(accessibleName);
+            '%d users', numMembers), numMembers);
+        this._showUserListButton.update_property(
+            [Gtk.AccessibleProperty.LABEL], [accessibleName]);
         this._showUserListButton.label = `${numMembers}`;
     }
 
