@@ -1,9 +1,3 @@
-export {
-    RoomList,
-    RoomListHeader,
-    RoomRow
-};
-
 import Gdk from 'gi://Gdk';
 import Gio from 'gi://Gio';
 import GLib from 'gi://GLib';
@@ -11,24 +5,23 @@ import GObject from 'gi://GObject';
 import Gtk from 'gi://Gtk';
 import Tp from 'gi://TelepathyGLib';
 
-import { AccountsMonitor } from './accountsMonitor.js';
-import { RoomManager } from './roomManager.js';
-import { UserStatusMonitor } from './userTracker.js';
+import AccountsMonitor from './accountsMonitor.js';
+import RoomManager from './roomManager.js';
+import UserStatusMonitor from './userTracker.js';
 
 const MIN_SPINNER_TIME = 1000000;   // in microsecond
 
 function _onPopoverVisibleChanged(popover) {
-    let context = popover.relative_to.get_style_context();
     if (popover.visible)
-        context.add_class('has-open-popup');
+        popover.get_parent().add_css_class('has-open-popup');
     else
-        context.remove_class('has-open-popup');
+        popover.get_parent().remove_css_class('has-open-popup');
 }
 
 const RoomRow = GObject.registerClass({
     Template: 'resource:///org/gnome/Polari/ui/room-list-row.ui',
     InternalChildren: [
-        'eventBox',
+        'box',
         'icon',
         'roomLabel',
         'counter',
@@ -55,10 +48,16 @@ const RoomRow = GObject.registerClass({
 
         this._icon.visible = room.icon !== null;
 
-        this._eventBox.connect('button-release-event',
-            this._onButtonRelease.bind(this));
-        this.connect('key-press-event',
-            this._onKeyPress.bind(this));
+        this._keyController = new Gtk.EventControllerKey();
+        this._keyController.connect('key-pressed', this._onKeyPressed.bind(this));
+        this.add_controller(this._keyController);
+
+        this._clickGesture = new Gtk.GestureClick({
+            button: Gdk.BUTTON_SECONDARY,
+        });
+        this._clickGesture.connect('released',
+            this._onButtonReleased.bind(this));
+        this._box.add_controller(this._clickGesture);
 
         room.bind_property('display-name',
             this._roomLabel, 'label',
@@ -177,11 +176,10 @@ const RoomRow = GObject.registerClass({
         this._counter.label = nHighlights.toString();
         this._counter.opacity = nHighlights > 0 ? 1. : 0.;
 
-        let context = this.get_style_context();
         if (nPending === 0)
-            context.add_class('inactive');
+            this.add_css_class('inactive');
         else
-            context.remove_class('inactive');
+            this.remove_css_class('inactive');
     }
 
     _onChannelChanged() {
@@ -200,19 +198,12 @@ const RoomRow = GObject.registerClass({
         this._updatePending();
     }
 
-    _onButtonRelease(w, event) {
-        let [, button] = event.get_button();
-        if (button !== Gdk.BUTTON_SECONDARY)
-            return Gdk.EVENT_PROPAGATE;
-
+    _onButtonReleased(controller) {
+        controller.set_state(Gtk.EventSequenceState.CLAIMED);
         this._showPopover();
-
-        return Gdk.EVENT_STOP;
     }
 
-    _onKeyPress(w, event) {
-        let [, keyval] = event.get_keyval();
-        let [, mods] = event.get_state();
+    _onKeyPressed(controller, keyval, keycode, mods) {
         if (keyval !== Gdk.KEY_Menu &&
             !(keyval === Gdk.KEY_F10 &&
               mods & Gdk.ModifierType.SHIFT_MASK))
@@ -237,12 +228,12 @@ const RoomRow = GObject.registerClass({
 });
 
 const RoomRowPopover = GObject.registerClass(
-class RoomRowPopover extends Gtk.Popover {
+class RoomRowPopover extends Gtk.PopoverMenu {
     _init(row) {
         super._init({
-            relative_to: row,
             position: Gtk.PositionType.BOTTOM,
         });
+        this.set_parent(row);
 
         this.connect('notify::visible', _onPopoverVisibleChanged);
 
@@ -262,15 +253,21 @@ class RoomRowPopover extends Gtk.Popover {
         const label = isRoom ?  _('Leave chatroom') : _('End conversation');
         this._menu.append(label, `app.leave-room(("${this._row.room.id}", ""))`);
 
-        this.bind_model(this._menu, null);
+        this.set_menu_model(this._menu);
     }
 
     vfunc_map() {
         if (this._row.room.type !== Tp.HandleType.ROOM)
             this._updateMuteItem();
+        this._previousFocus = this.get_root().get_focus();
         super.vfunc_map();
     }
 
+    vfunc_unmap() {
+        this._previousFocus.grab_focus();
+        super.vfunc_unmap();
+    }
+
     _updateMuteItem() {
         this._menu.remove(0);
 
@@ -304,16 +301,56 @@ const RoomListHeader = GObject.registerClass({
         'popoverProperties',
         'spinner',
     ],
-}, class RoomListHeader extends Gtk.MenuButton {
+    Properties: {
+        'popover': GObject.ParamSpec.object(
+            'popover', 'popover', 'popover',
+            GObject.ParamFlags.READWRITE,
+            Gtk.Popover.$gtype),
+    },
+    Signals: {
+        'activate': { flags: GObject.SignalFlags.ACTION },
+    },
+}, class RoomListHeader extends Gtk.Widget {
+    static _classInit(klass) {
+        klass = Gtk.Widget._classInit(klass);
+
+        Gtk.Widget.set_activate_signal_from_name.call(klass, 'activate');
+        Gtk.Widget.set_layout_manager_type = Gtk.GridLayout;
+
+        return klass;
+    }
+
     _init(params) {
         this._account = params.account;
         delete params.account;
 
-        params.name = `RoomListHeader ${this._account.display_name}`;
-
         this._app = Gio.Application.get_default();
+        this._popover = null;
 
-        super._init(params);
+        super._init({
+            ...params,
+            name: `RoomListHeader ${this._account.display_name}`,
+        });
+
+        this.connect('activate',
+            () => this._popover?.popup());
+
+        this._clickGesture = new Gtk.GestureClick({
+            propagation_phase: Gtk.PropagationPhase.CAPTURE,
+            button: 0,
+        });
+        this.add_controller(this._clickGesture);
+
+        this._clickGesture.connect('released', () => {
+            const button = this._clickGesture.get_current_button();
+            if (button !== Gdk.BUTTON_PRIMARY && button !== Gdk.BUTTON_SECONDARY)
+                return;
+
+            this._clickGesture.set_state(Gtk.EventSequenceState.CLAIMED);
+
+            this._previousFocus = this.get_root().get_focus();
+            this._popover?.popup();
+        });
 
         this.popover.name = `ConnectionPopover ${this._account.display_name}`;
 
@@ -321,6 +358,7 @@ const RoomListHeader = GObject.registerClass({
         this.popover.connect('notify::visible', _onPopoverVisibleChanged);
         this.popover.connect('closed', () => {
             this._popoverPassword.text = '';
+            this._previousFocus?.grab_focus();
         });
 
         let target = new GLib.Variant('o', this._account.get_object_path());
@@ -367,6 +405,30 @@ const RoomListHeader = GObject.registerClass({
         });
     }
 
+    vfunc_unroot() {
+        [...this].forEach(child => child.unparent());
+
+        super.vfunc_unroot();
+    }
+
+    get popover() {
+        return this._popover;
+    }
+
+    set popover(popover) {
+        if (this._popover === popover)
+            return;
+
+        this._popover?.unparent();
+        this._popover?.run_dispose();
+
+        this._popover = popover;
+
+        this._popover?.set_parent(this);
+
+        this.notify('popover');
+    }
+
     _onDisplayNameChanged() {
         this._label.label = this._account.display_name;
 
@@ -381,21 +443,9 @@ const RoomListHeader = GObject.registerClass({
         if (parent)
             parent.invalidate_sort();
 
-        let accessibleName = _('Network %s has an error').format(this._account.display_name);
-        this.get_accessible().set_name(accessibleName);
-    }
-
-    /* hack: Handle primary and secondary button interchangeably */
-    vfunc_button_press_event(event) {
-        if (event.button === Gdk.BUTTON_SECONDARY)
-            event.button = Gdk.BUTTON_PRIMARY;
-        return super.vfunc_button_press_event(event);
-    }
-
-    vfunc_button_release_event(event) {
-        if (event.button === Gdk.BUTTON_SECONDARY)
-            event.button = Gdk.BUTTON_PRIMARY;
-        return super.vfunc_button_release_event(event);
+        let accessibleName = vprintf(_('Network %s has an error'), this._account.display_name);
+        this.update_property(
+            [Gtk.AccessibleProperty.LABEL], [accessibleName]);
     }
 
     _getConnectionStatus() {
@@ -421,7 +471,7 @@ const RoomListHeader = GObject.registerClass({
         else if (status === Tp.ConnectionStatus.DISCONNECTED)
             child = 'disconnected';
 
-        if (isError && this._spinner.active) {
+        if (isError && this._spinner.spinning) {
             let spinnerTime = GLib.get_monotonic_time() - this._spinnerActivationTime;
             if (spinnerTime < MIN_SPINNER_TIME) {
                 GLib.timeout_add(GLib.PRIORITY_DEFAULT, (MIN_SPINNER_TIME - spinnerTime) / 1000, () => {
@@ -433,18 +483,17 @@ const RoomListHeader = GObject.registerClass({
         }
 
         this._iconStack.visible_child_name = child;
-        this._spinner.active = child === 'connecting';
+        this._spinner.spinning = child === 'connecting';
         this._popoverTitle.visible = !isAuth;
 
-        if (this._spinner.active)
+        if (this._spinner.spinning)
             this._spinnerActivationTime = GLib.get_monotonic_time();
 
         this._popoverTitle.use_markup = isError;
         this._popoverStatus.use_markup = !isError;
 
         if (!isError) {
-            let styleContext = this._popoverStatus.get_style_context();
-            styleContext.add_class('dim-label');
+            this._popoverStatus.add_css_class('dim-label');
 
             let params = this._account.dup_parameters_vardict().deep_unpack();
             let server = params['server'].deep_unpack();
@@ -452,12 +501,11 @@ const RoomListHeader = GObject.registerClass({
 
             /* Translators: This is an account name followed by a
                server address, e.g. "GNOME (irc.gnome.org)" */
-            let fullTitle = _('%s (%s)').format(accountName, server);
+            let fullTitle = vprintf(_('%s (%s)'), accountName, server);
             this._popoverTitle.label = accountName === server ? accountName : fullTitle;
             this._popoverStatus.label = `<sup>${this._getStatusLabel()}<${'/'}sup>`;
         } else {
-            let styleContext = this._popoverStatus.get_style_context();
-            styleContext.remove_class('dim-label');
+            this._popoverStatus.remove_css_class('dim-label');
 
             this._popoverTitle.label = `<b>${_('Connection Problem')}<${'/'}b>`;
             this._popoverStatus.label = this._getErrorLabel();
@@ -501,24 +549,24 @@ const RoomListHeader = GObject.registerClass({
         case Tp.error_get_dbus_name(Tp.Error.CERT_HOSTNAME_MISMATCH):
         case Tp.error_get_dbus_name(Tp.Error.CERT_FINGERPRINT_MISMATCH):
         case Tp.error_get_dbus_name(Tp.Error.CERT_SELF_SIGNED):
-            return _('Could not connect to %s in a safe way.').format(this._account.display_name);
+            return vprintf(_('Could not connect to %s in a safe way.'), this._account.display_name);
 
         case Tp.error_get_dbus_name(Tp.Error.AUTHENTICATION_FAILED):
-            return _('%s requires a password.').format(this._account.display_name);
+            return vprintf(_('%s requires a password.'), this._account.display_name);
 
         case Tp.error_get_dbus_name(Tp.Error.CONNECTION_FAILED):
         case Tp.error_get_dbus_name(Tp.Error.CONNECTION_LOST):
         case Tp.error_get_dbus_name(Tp.Error.CONNECTION_REPLACED):
         case Tp.error_get_dbus_name(Tp.Error.SERVICE_BUSY):
-            return _('Could not connect to %s. The server is busy.').format(this._account.display_name);
+            return vprintf(_('Could not connect to %s. The server is busy.'), this._account.display_name);
 
         default:
-            return _('Could not connect to %s.').format(this._account.display_name);
+            return vprintf(_('Could not connect to %s.'), this._account.display_name);
         }
     }
 });
 
-const RoomList = GObject.registerClass(
+export default GObject.registerClass(
 class RoomList extends Gtk.ListBox {
     _init(params) {
         super._init(params);
@@ -595,12 +643,22 @@ class RoomList extends Gtk.ListBox {
     vfunc_realize() {
         super.vfunc_realize();
 
-        let toplevel = this.get_toplevel();
-        toplevel.connect('notify::active-room',
-            this._activeRoomChanged.bind(this));
+        const toplevel = this.get_root();
+        this._toplevelSignals = [
+            toplevel.connect('notify::active-room',
+                () => this._activeRoomChanged()),
+        ];
         this._activeRoomChanged();
     }
 
+    vfunc_unrealize() {
+        super.vfunc_unrealize();
+
+        const toplevel = this.get_root();
+        this._toplevelSignals.forEach(id => toplevel.disconnect(id));
+        this._toplevelSignals = [];
+    }
+
     _rowToRoomIndex(index) {
         let placeholders = [...this._placeholders.values()];
         let nBefore = placeholders.filter(p => p.get_index() < index).length;
@@ -608,7 +666,7 @@ class RoomList extends Gtk.ListBox {
     }
 
     _roomToRowIndex(index) {
-        let roomRows = this.get_children().filter(r => r instanceof RoomRow);
+        let roomRows = [...this].filter(r => r instanceof RoomRow);
         return index < roomRows.length ? roomRows[index].get_index() : -1;
     }
 
@@ -649,7 +707,7 @@ class RoomList extends Gtk.ListBox {
         if (this._roomManager.roomCount === 0)
             return;
 
-        let toplevel = this.get_toplevel();
+        const toplevel = this.get_root();
         let current = this._roomRows.get(toplevel.active_room.id);
 
         if (current !== row)
@@ -679,12 +737,12 @@ class RoomList extends Gtk.ListBox {
         let placeholder = new Gtk.ListBoxRow({
             selectable: false,
             activatable: false,
-            no_show_all: true,
+            visible: false,
         });
         placeholder.account = account;
 
         this._placeholders.set(account, placeholder);
-        this.add(placeholder);
+        this.append(placeholder);
 
         placeholder.connect('notify::visible', () => {
             this.invalidate_sort();
@@ -700,7 +758,8 @@ class RoomList extends Gtk.ListBox {
             return;
 
         this._placeholders.delete(account);
-        placeholder.destroy();
+        this.remove(placeholder);
+        placeholder.run_dispose();
     }
 
     _roomAdded(roomManager, room) {
@@ -708,7 +767,7 @@ class RoomList extends Gtk.ListBox {
             return;
 
         let row = new RoomRow(room);
-        this.add(row);
+        this.append(row);
         this._roomRows.set(room.id, row);
 
         row.connect('destroy', w => this._roomRows.delete(w.room.id));
@@ -721,7 +780,8 @@ class RoomList extends Gtk.ListBox {
             return;
 
         this._moveSelectionFromRow(row);
-        row.destroy();
+        this.remove(row);
+        row.run_dispose();
         this._roomRows.delete(room.id);
         this._updatePlaceholderVisibility(room.account);
     }
@@ -738,7 +798,7 @@ class RoomList extends Gtk.ListBox {
     }
 
     _activeRoomChanged() {
-        let room = this.get_toplevel().active_room;
+        const room = this.get_root().active_room;
         if (!room)
             return;
         let row = this._roomRows.get(room.id);
@@ -751,7 +811,7 @@ class RoomList extends Gtk.ListBox {
     }
 
     on_row_selected(row) {
-        this.get_toplevel().active_room = row ? row.room : null;
+        this.get_root().active_room = row ? row.room : null;
         if (row)
             row.selected();
     }
@@ -763,8 +823,9 @@ class RoomList extends Gtk.ListBox {
         let oldHeader = row.get_header();
 
         if (beforeAccount === account) {
-            if (oldHeader)
-                oldHeader.destroy();
+            row.set_header(null);
+            oldHeader?.unparent();
+            oldHeader?.run_dispose();
             return;
         }
 
