@@ -1,23 +1,25 @@
-export { RoomStack };
-
 import Gio from 'gi://Gio';
 import GLib from 'gi://GLib';
 import GObject from 'gi://GObject';
 import Gtk from 'gi://Gtk';
 import Tp from 'gi://TelepathyGLib';
 
-import { AccountsMonitor } from './accountsMonitor.js';
-import { ChatView } from './chatView.js';
-import { EntryArea } from './entryArea.js';
+import AccountsMonitor from './accountsMonitor.js';
+import ChatView from './chatView.js';
+import EntryArea from './entryArea.js';
 import { MessageInfoBar } from './appNotifications.js';
-import { RoomManager } from './roomManager.js';
+import RoomManager from './roomManager.js';
 
-const RoomStack = GObject.registerClass({
+export default GObject.registerClass({
     Properties: {
         'entry-area-height': GObject.ParamSpec.uint(
             'entry-area-height', 'entry-area-height', 'entry-area-height',
             GObject.ParamFlags.READABLE,
             0, GLib.MAXUINT32, 0),
+        'view-height': GObject.ParamSpec.uint(
+            'view-height', 'view-height', 'view-height',
+            GObject.ParamFlags.READABLE,
+            0, GLib.MAXUINT32, 0),
     },
 }, class RoomStack extends Gtk.Stack {
     _init(params) {
@@ -37,12 +39,7 @@ const RoomStack = GObject.registerClass({
         this.add_named(new ChatPlaceholder(this._sizeGroup), 'placeholder');
 
         this._entryAreaHeight = 0;
-        this._sizeGroup.get_widgets()[0].connect('size-allocate', (w, rect) => {
-            if (this._entryAreaHeight !== rect.height - 1) {
-                this._entryAreaHeight = rect.height - 1;
-                this.notify('entry-area-height');
-            }
-        });
+        this._viewHeight = 0;
 
         this.connect('destroy', () => {
             this._roomManager.disconnect(this._roomAddedId);
@@ -53,21 +50,55 @@ const RoomStack = GObject.registerClass({
     vfunc_realize() {
         super.vfunc_realize();
 
-        let toplevel = this.get_toplevel();
+        const toplevel = this.get_root();
 
-        toplevel.connect('notify::active-room',
-            this._activeRoomChanged.bind(this));
-        toplevel.connect('active-room-state-changed',
-            this._updateSensitivity.bind(this));
+        this._toplevelSignals = [
+            toplevel.connect('notify::active-room',
+                this._activeRoomChanged.bind(this)),
+            toplevel.connect('active-room-state-changed',
+                this._updateSensitivity.bind(this)),
+        ];
         this._activeRoomChanged();
         this._updateSensitivity();
     }
 
+    vfunc_unrealize() {
+        super.vfunc_unrealize();
+
+        const toplevel = this.get_root();
+        this._toplevelSignals.forEach(id => toplevel.disconnect(id));
+        this._toplevelSignals = [];
+    }
+
+    vfunc_size_allocate(width, height, baseline) {
+        super.vfunc_size_allocate(width, height, baseline);
+
+        const [firstEntry] =
+            this._sizeGroup.get_widgets().filter(w => w.get_mapped());
+        const entryHeight = firstEntry
+            ? firstEntry.get_allocated_height() - 1 : 0;
+        if (this._entryAreaHeight !== entryHeight) {
+            this._entryAreaHeight = entryHeight;
+            this.notify('entry-area-height');
+        }
+
+        const viewHeight = this.get_allocated_height() - entryHeight;
+        if (this._viewHeight !== viewHeight) {
+            this._viewHeight = viewHeight;
+            this.notify('view-height');
+        }
+    }
+
     // eslint-disable-next-line camelcase
     get entry_area_height() {
         return this._entryAreaHeight;
     }
 
+    // eslint-disable-next-line camelcase
+    get view_height() {
+        return this._viewHeight;
+    }
+
     _addView(id, view) {
         this._rooms.set(id, view);
         this.add_named(view, id);
@@ -78,17 +109,20 @@ const RoomStack = GObject.registerClass({
     }
 
     _roomRemoved(roomManager, room) {
-        this._rooms.get(room.id).destroy();
+        const view = this._rooms.get(room.id);
         this._rooms.delete(room.id);
+
+        this.remove(view);
+        view.run_dispose();
     }
 
     _activeRoomChanged() {
-        let room = this.get_toplevel().active_room;
+        const room = this.get_root().active_room;
         this.set_visible_child_name(room ? room.id : 'placeholder');
     }
 
     _updateSensitivity() {
-        let room = this.get_toplevel().active_room;
+        const room = this.get_root().active_room;
         if (!room)
             return;
         let sensitive = room && room.channel;
@@ -102,9 +136,9 @@ class SavePasswordConfirmationBar extends MessageInfoBar {
         this._room = room;
 
         let title = _('Should the password be saved?');
-        let subtitle =
-            _('Identification will happen automatically the next time you connect to %s')
-            .format(this._room.account.display_name);
+        let subtitle = vprintf(
+            _('Identification will happen automatically the next time you connect to %s'),
+            this._room.account.display_name);
         super._init({ title, subtitle });
 
         this.connect('destroy', this._onDestroy.bind(this));
@@ -206,14 +240,14 @@ class ChatPlaceholder extends Gtk.Overlay {
             margin_start: 14,
         });
         title.label = `<span letter_spacing="4500">${_('Polari')}<${'/'}span>`;
-        title.get_style_context().add_class('polari-background-title');
+        title.add_css_class('polari-background-title');
 
         let description = new Gtk.Label({
             label: _('Join a room using the + button.'),
             halign: Gtk.Align.CENTER, wrap: true,
             margin_top: 24, use_markup: true,
         });
-        description.get_style_context().add_class('polari-background-description');
+        description.add_css_class('polari-background-description');
 
         let inputPlaceholder = new Gtk.Box({ valign: Gtk.Align.END });
         sizeGroup.add_widget(inputPlaceholder);
@@ -227,13 +261,12 @@ class ChatPlaceholder extends Gtk.Overlay {
             vexpand: true,
             valign: Gtk.Align.CENTER,
         });
-        grid.get_style_context().add_class('polari-background');
+        grid.add_css_class('polari-background');
         grid.attach(image, 0, 0, 1, 1);
         grid.attach(title, 1, 0, 1, 1);
         grid.attach(description, 0, 1, 2, 1);
-        this.add(grid);
+        this.set_child(grid);
         this.add_overlay(inputPlaceholder);
-        this.show_all();
     }
 });
 
@@ -243,7 +276,7 @@ class RoomView extends Gtk.Overlay {
         super._init({ name: `RoomView ${room.display_name}` });
 
         let box = new Gtk.Box({ orientation: Gtk.Orientation.VERTICAL });
-        this.add(box);
+        this.set_child(box);
 
         if (room.type === Tp.HandleType.CONTACT)
             this.add_overlay(new SavePasswordConfirmationBar(room));
@@ -251,13 +284,13 @@ class RoomView extends Gtk.Overlay {
         this.add_overlay(new ChannelErrorBar(room));
 
         this._view = new ChatView(room);
-        box.add(this._view);
+        box.append(this._view);
 
         this._entryArea = new EntryArea({
             room,
             sensitive: false,
         });
-        box.add(this._entryArea);
+        box.append(this._entryArea);
 
         this._view.bind_property('max-nick-chars',
             this._entryArea, 'max-nick-chars',
@@ -273,8 +306,6 @@ class RoomView extends Gtk.Overlay {
         this._view.connect('file-dropped', (view, file) => {
             this._entryArea.pasteFile(file);
         });
-
-        this.show_all();
     }
 
     set inputSensitive(sensitive) {
