@@ -1,14 +1,12 @@
-export { TelepathyClient };
-
 import Gio from 'gi://Gio';
 import GLib from 'gi://GLib';
 import GObject from 'gi://GObject';
 import Polari from 'gi://Polari';
 import Tp from 'gi://TelepathyGLib';
 
-import { AccountsMonitor } from './accountsMonitor.js';
-import { RoomManager } from './roomManager.js';
-import { UserStatusMonitor } from './userTracker.js';
+import AccountsMonitor from './accountsMonitor.js';
+import RoomManager from './roomManager.js';
+import UserStatusMonitor from './userTracker.js';
 import * as Utils from './utils.js';
 
 Gio._promisify(Gio.DBusConnection.prototype, 'call', 'call_finish');
@@ -29,24 +27,25 @@ Gio._promisify(Tp.TextChannel.prototype,
 
 const SHELL_CLIENT_PREFIX = 'org.freedesktop.Telepathy.Client.GnomeShell';
 
-const SASLAuthenticationIface = '<node> \
-<interface name="org.freedesktop.Telepathy.Channel.Interface.SASLAuthentication"> \
-<method name="StartMechanismWithData"> \
-    <arg type="s" direction="in" name="mechanism" /> \
-    <arg type="ay" direction="in" name="data" /> \
-</method> \
-<method name="AcceptSASL"/> \
-<method name="AbortSASL"> \
-    <arg type="u" direction="in" name="reason"/> \
-    <arg type="s" direction="in" name="debug-message"/> \
-</method> \
-<signal name="SASLStatusChanged"> \
-    <arg name="status" type="u" /> \
-    <arg name="reason" type="s" /> \
-    <arg name="details" type="a{sv}" /> \
-</signal> \
-</interface> \
-</node>';
+const SASLAuthenticationIface = `
+<node>
+<interface name="org.freedesktop.Telepathy.Channel.Interface.SASLAuthentication">
+  <method name="StartMechanismWithData">
+    <arg type="s" direction="in" name="mechanism" />
+    <arg type="ay" direction="in" name="data" />
+  </method>
+  <method name="AcceptSASL"/>
+  <method name="AbortSASL">
+    <arg type="u" direction="in" name="reason"/>
+    <arg type="s" direction="in" name="debug-message"/>
+  </method>
+  <signal name="SASLStatusChanged">
+    <arg name="status" type="u" />
+    <arg name="reason" type="s" />
+    <arg name="details" type="a{sv}" />
+  </signal>
+</interface>
+</node>`;
 let SASLAuthProxy = Gio.DBusProxy.makeProxyWrapper(SASLAuthenticationIface);
 
 const SASLStatus = {
@@ -94,7 +93,7 @@ class SASLAuthHandler {
     _onSASLStatusChanged(proxy, sender, [status]) {
         let name = this._channel.connection.get_account().display_name;
         let statusString = Object.keys(SASLStatus)[status];
-        debug(`Auth status for server ${name}: ${statusString}`);
+        console.info(`Auth status for server ${name}: ${statusString}`);
 
         switch (status) {
         case SASLStatus.NOT_STARTED:
@@ -124,7 +123,7 @@ class SASLAuthHandler {
     }
 }
 
-const TelepathyClient = GObject.registerClass(
+export default GObject.registerClass(
 class TelepathyClient extends Tp.BaseClient {
     _init(params) {
         this._app = Gio.Application.get_default();
@@ -189,7 +188,8 @@ class TelepathyClient extends Tp.BaseClient {
                 null);
             [names] = result.deep_unpack();
         } catch (e) {
-            debug(`Failed to list bus names: ${e}`);
+            console.info('Failed to list bus names');
+            console.debug(e);
         }
 
         this._shellHandlesPrivateChats =
@@ -276,7 +276,7 @@ class TelepathyClient extends Tp.BaseClient {
         let presence = account.reachable
             ? Tp.ConnectionPresenceType.AVAILABLE
             : Tp.ConnectionPresenceType.OFFLINE;
-        debug(`Account ${account.display_name} is now ${account.reachable
+        console.info(`Account ${account.display_name} is now ${account.reachable
             ? 'reachable'
             : 'unreachable'}`);
 
@@ -289,8 +289,9 @@ class TelepathyClient extends Tp.BaseClient {
 
         try {
             const password = await Utils.lookupIdentifyPassword(account);
-            this._sendIdentify(account, password);
+            await this._sendIdentify(account, password);
         } catch (e) {
+            console.debug(e);
             this._connectRooms(account);
         }
     }
@@ -309,11 +310,13 @@ class TelepathyClient extends Tp.BaseClient {
         let msg = account.requested_status_message;
         let accountName = account.display_name;
 
-        debug(`Setting presence of account "${accountName}" to ${status}`);
+        console.info(`Setting presence of account "${accountName}" to ${status}`);
         try {
             await account.request_presence_async(presence, status, msg);
         } catch (e) {
-            log(`Connection failed: ${e.message}`);
+            console.warn(`Failed to change presence of account "${
+                accountName}" to ${status}`);
+            console.debug(e);
         }
     }
 
@@ -356,7 +359,9 @@ class TelepathyClient extends Tp.BaseClient {
             channel = await req.ensure_and_observe_channel_async(
                 preferredHandler, cancellable);
         } catch (e) {
-            debug(`Failed to ensure channel: ${e.message}`);
+            console.warn(`Failed to ensure channel ${
+                targetId} for account ${account.displayName}`);
+            console.debug(e);
             channelError = Tp.error_get_dbus_name(e.code);
             throw e;
         } finally {
@@ -397,24 +402,13 @@ class TelepathyClient extends Tp.BaseClient {
         try {
             await room.send_identify_message_async(command, username, password);
         } catch (e) {
-            log(`Failed to send identify message: ${e.message}`);
+            console.warn(`Failed to send identify message for ${
+                username} to ${contactName} on ${account.displayName}`);
+            console.debug(e);
         }
         this._connectRooms(account);
     }
 
-    async _sendMessage(channel, message) {
-        if (!message || !channel)
-            return;
-
-        let type = Tp.ChannelTextMessageType.NORMAL;
-        try {
-            await channel.send_message_async(
-                Tp.ClientMessage.new_text(type, message), 0);
-        } catch (e) {
-            log(`Failed to send message: ${e.message}`);
-        }
-    }
-
     _onConnectAccountActivated(action, parameter) {
         let accountPath = parameter.deep_unpack();
         let account = this._accountsMonitor.lookupAccount(accountPath);
@@ -449,7 +443,7 @@ class TelepathyClient extends Tp.BaseClient {
     }
 
     async _onQueryActivated(action, parameter) {
-        let [accountPath, channelName, message, time_] = parameter.deep_unpack();
+        let [accountPath, channelName, message, present_] = parameter.deep_unpack();
         let account = this._accountsMonitor.lookupAccount(accountPath);
 
         if (!account || !account.enabled)
@@ -466,8 +460,11 @@ class TelepathyClient extends Tp.BaseClient {
             let tpMessage = Tp.ClientMessage.new_text(type, message);
             await channel.send_message_async(tpMessage, 0);
         } catch (e) {
-            if (message)
-                log(`Failed to send message: ${e.message}`);
+            if (!message)
+                return; // already logged by _requestChannel()
+            console.warn(`Failed to send message to ${channelName} on ${
+                account.displayName}`);
+            console.debug(e);
         }
     }
 
@@ -497,7 +494,9 @@ class TelepathyClient extends Tp.BaseClient {
         try {
             await room.channel.leave_async(reason, message);
         } catch (e) {
-            log(`Failed to leave channel: ${e.message}`);
+            console.warn(`Failed to leave channel ${
+                room.channelName} on ${room.account.displayName}`);
+            console.debug(e);
         }
     }
 
@@ -607,7 +606,7 @@ class TelepathyClient extends Tp.BaseClient {
             channel.connect('pending-message-removed',
                 this._onPendingMessageRemoved.bind(this));
 
-            this._roomManager.ensureRoomForChannel(channel, 0);
+            this._roomManager.ensureRoomForChannel(channel, false);
         });
     }
 
@@ -624,7 +623,7 @@ class TelepathyClient extends Tp.BaseClient {
             if (present)
                 this._app.activate();
 
-            this._roomManager.ensureRoomForChannel(channel, time);
+            this._roomManager.ensureRoomForChannel(channel, present);
             // channel.join_async('', null);
         });
     }
@@ -645,16 +644,16 @@ class TelepathyClient extends Tp.BaseClient {
         let params = [
             room.account.object_path,
             room.channel_name,
-            Utils.getTpEventTime(),
+            true,
         ];
 
         let actionName, paramFormat;
         if (room.type === Tp.HandleType.ROOM) {
             actionName = 'app.join-room';
-            paramFormat = '(ssu)';
+            paramFormat = '(ssb)';
         } else {
             actionName = 'app.message-user';
-            paramFormat = '(sssu)';
+            paramFormat = '(sssb)';
             params.splice(2, 0, '');
         }
 
@@ -681,8 +680,9 @@ class TelepathyClient extends Tp.BaseClient {
         let accountName = room.account.display_name;
         /* Translators: Those are a botname and an accountName, e.g.
            "Save NickServ password for GNOME" */
-        let summary = _('Save %s password for %s?').format(data.botname, accountName);
-        let text = _('Identification will happen automatically the next time you connect to %s').format(accountName);
+        let summary = vprintf(_('Save %s password for %s?'), data.botname, accountName);
+        let text = vprintf(
+            _('Identification will happen automatically the next time you connect to %s'), accountName);
         let notification = this._createNotification(room, summary, text);
 
         notification.add_button_with_target(_('Save'),
@@ -717,11 +717,11 @@ class TelepathyClient extends Tp.BaseClient {
         let summary;
 
         if (room.type === Tp.HandleType.CONTACT) {
-            summary = '%s'.format(nick);
+            summary = vprintf('%s', nick);
         } else {
             /* Translators: This is the title of the notification announcing a newly
                received message, in the form "user-nickname in room-display-name" */
-            summary = _('%s in %s').format(nick, room.display_name);
+            summary = vprintf(_('%s in %s'), nick, room.display_name);
         }
 
         let notification = this._createNotification(room, summary, text);
