@@ -4,7 +4,18 @@ import Gio from 'gi://Gio';
 import GLib from 'gi://GLib';
 import GObject from 'gi://GObject';
 import Gtk from 'gi://Gtk?version=3.0';
-import WebKit2 from 'gi://WebKit2?version=4.0';
+
+import { setConsoleLogDomain } from 'console';
+import { programArgs } from 'system';
+
+import gi from 'gi';
+let WebKit2;
+
+try {
+    WebKit2 = gi.require('WebKit2', '4.1');
+} catch (e) {
+    WebKit2 = gi.require('WebKit2', '4.0');
+}
 
 Gio._promisify(WebKit2.WebView.prototype, 'get_snapshot', 'get_snapshot_finish');
 Gio._promisify(WebKit2.WebView.prototype, 'run_javascript', 'run_javascript_finish');
@@ -84,7 +95,8 @@ let PreviewWindow = GObject.registerClass({
             clip = await getClipOp;
             snapshot = await snapshotOp;
         } catch (e) {
-            log(`Creating snapshot failed: ${e}`);
+            console.warn(`Failed to create snapshot of ${this.uri}`);
+            console.debug(e);
             this.emit('snapshot-failed');
             return;
         }
@@ -111,7 +123,8 @@ let PreviewWindow = GObject.registerClass({
             let res = await this._view.run_javascript(script, null);
             obj = res.get_js_value();
         } catch (e) {
-            log(`Failed to get clip information: ${e} (${e.code})`);
+            console.warn(`Failed to get clip information from ${this.uri}`);
+            console.debug(e);
         }
 
         if (!obj || obj.is_null())
@@ -153,6 +166,11 @@ class App {
     run() {
         Gtk.init(null);
 
+        const logDomain = 'Polari Thumbnailer';
+        setConsoleLogDomain(logDomain);
+        if (GLib.log_writer_is_journald(2))
+            GLib.setenv('G_MESSAGES_DEBUG', logDomain, false);
+
         let window = new PreviewWindow({
             uri: this._uri,
             default_width: 10 * PREVIEW_WIDTH,
@@ -220,6 +238,6 @@ class App {
     }
 }
 
-let [url, filename] = ARGV;
+let [url, filename] = programArgs;
 let app = new App(url, filename);
 app.run();
