@@ -1,5 +1,4 @@
-export { URLPreview };
-
+import GdkPixbuf from 'gi://GdkPixbuf';
 import Gio from 'gi://Gio';
 import GLib from 'gi://GLib';
 import GObject from 'gi://GObject';
@@ -100,7 +99,7 @@ class Thumbnailer {
     }
 }
 
-const URLPreview = GObject.registerClass({
+export default GObject.registerClass({
     Properties: {
         'uri': GObject.ParamSpec.string(
             'uri', 'uri', 'uri',
@@ -116,24 +115,20 @@ const URLPreview = GObject.registerClass({
             spacing: 6,
         });
 
-        let styleContext = this.get_style_context();
-        styleContext.add_class('url-preview');
-        styleContext.add_class('background');
+        this.add_css_class('url-preview');
+        this.add_css_class('background');
 
         this._imageLoaded = false;
-        this._image = new Gtk.Image({
-            visible: true,
-        });
-        this._image.get_style_context().add_class('dim-label');
-        this.add(this._image);
+        this._image = new Gtk.Image();
+        this._image.add_css_class('dim-label');
+        this.append(this._image);
 
         this._label = new Gtk.Label({
             halign: Gtk.Align.START,
             ellipsize: Pango.EllipsizeMode.END,
-            visible: true,
         });
-        this._label.get_style_context().add_class('dim-label');
-        this.add(this._label);
+        this._label.add_css_class('dim-label');
+        this.append(this._label);
 
         this._networkMonitor = Gio.NetworkMonitor.get_default();
         this._networkChangedId = this._networkMonitor.connect('network-changed',
@@ -159,24 +154,27 @@ const URLPreview = GObject.registerClass({
         });
         const thumbnailer = Thumbnailer.getDefault();
 
+        let title;
         try {
             const filename = await thumbnailer.getThumbnail(this.uri);
-            this._image.set_from_file(filename);
+            const pixbuf = GdkPixbuf.Pixbuf.new_from_file(filename);
+
+            title = pixbuf.get_option('tEXt::Title');
+            this._image.set_from_pixbuf(pixbuf);
+            this._image.remove_css_class('dim-label');
         } catch (e) {
-            if (e.matches(Gio.IOErrorEnum, Gio.IOErrorEnum.NETWORK_UNREACHABLE))
+            if (e.matches(Gio.IOErrorEnum, Gio.IOErrorEnum.NETWORK_UNREACHABLE)) {
                 this._imageLoaded = false;
-            else
-                log(`Thumbnail generation for ${this.uri} failed: ${e}`);
+            } else {
+                console.info(`Failed to generate thumbnail for ${this.uri}`);
+                console.debug(e);
+            }
             this._image.set({
                 icon_name: 'image-x-generic-symbolic',
                 pixel_size: 64,
             });
         }
 
-        let title = null;
-        if (this._image.pixbuf)
-            title = this._image.pixbuf.get_option('tEXt::Title');
-
         if (title) {
             this._label.set_label(title);
             this.tooltip_text = title;
