@@ -1,10 +1,3 @@
-export {
-    UserList,
-    UserListPopover,
-    UserDetails,
-    UserPopover
-};
-
 import Gio from 'gi://Gio';
 import GLib from 'gi://GLib';
 import GObject from 'gi://GObject';
@@ -18,7 +11,7 @@ import * as Utils from './utils.js';
 const FILTER_ENTRY_THRESHOLD = 8;
 const MAX_USERS_WIDTH_CHARS = 17;
 
-const UserListPopover = GObject.registerClass(
+export const UserListPopover = GObject.registerClass(
 class UserListPopover extends Gtk.Popover {
     _init(params) {
         super._init(params);
@@ -39,7 +32,7 @@ class UserListPopover extends Gtk.Popover {
     vfunc_realize() {
         super.vfunc_realize();
 
-        let toplevel = this.get_toplevel();
+        const toplevel = this.get_root();
         toplevel.connect('notify::active-room',
             this._activeRoomChanged.bind(this));
         toplevel.connect('notify::view-height',
@@ -50,28 +43,23 @@ class UserListPopover extends Gtk.Popover {
         this._box = new Gtk.Box({
             orientation: Gtk.Orientation.VERTICAL,
             spacing: 6,
-            margin_start: 6,
-            margin_end: 6,
-            margin_top: 6,
-            margin_bottom: 6,
         });
-        this.add(this._box);
+        this.set_child(this._box);
 
         this._revealer = new Gtk.Revealer();
-        this._box.add(this._revealer);
+        this._box.append(this._revealer);
 
         this._entry = new Gtk.SearchEntry();
         this._entry.connect('search-changed', this._updateFilter.bind(this));
-        this._revealer.add(this._entry);
-
-        this._box.show_all();
+        this._revealer.set_child(this._entry);
     }
 
     _activeRoomChanged() {
         this._entry.text = '';
 
         if (this._userList)
-            this._userList.destroy();
+            this._box.remove(this._userList);
+        this._userList?.run_dispose();
         this._userList = null;
     }
 
@@ -81,9 +69,9 @@ class UserListPopover extends Gtk.Popover {
         if (!this.get_mapped())
             return;
 
-        let viewHeight = this.get_toplevel().view_height;
-        let [popoverHeight] = this.get_preferred_height();
-        let [userListHeight] = this._userList.get_preferred_height();
+        const viewHeight = this.get_root().view_height;
+        const [popoverHeight] = this.measure(Gtk.Orientation.VERTICAL, -1);
+        const [userListHeight] = this._userList.measure(Gtk.Orientation.VERTICAL, -1);
         let chromeHeight = popoverHeight - userListHeight;
         this._userList.max_content_height = viewHeight - chromeHeight;
     }
@@ -92,12 +80,12 @@ class UserListPopover extends Gtk.Popover {
         if (this._userList)
             return;
 
-        let room = this.get_toplevel().active_room;
+        const room = this.get_root().active_room;
         if (!room || room.type !== Tp.HandleType.ROOM)
             return;
 
         this._userList = new UserList(room);
-        this._box.add(this._userList);
+        this._box.append(this._userList);
 
         this._userList.vadjustment.connect('changed',
             this._updateEntryVisibility.bind(this));
@@ -121,7 +109,7 @@ class UserListPopover extends Gtk.Popover {
     }
 });
 
-const UserDetails = GObject.registerClass({
+export const UserDetails = GObject.registerClass({
     Template: 'resource:///org/gnome/Polari/ui/user-details.ui',
     InternalChildren: [
         'spinnerBox',
@@ -142,7 +130,7 @@ const UserDetails = GObject.registerClass({
             GObject.ParamFlags.READWRITE,
             false),
     },
-}, class UserDetails extends Gtk.Frame {
+}, class UserDetails extends Gtk.Box {
     _init(params = {}) {
         let { user } = params;
         delete params.user;
@@ -296,12 +284,11 @@ const UserDetails = GObject.registerClass({
 
         let app = Gio.Application.get_default();
         let action = app.lookup_action('message-user');
-        let time = Gtk.get_current_event().get_time();
-        action.activate(GLib.Variant.new('(sssu)', [
+        action.activate(GLib.Variant.new('(sssb)', [
             account.get_object_path(),
             this._user.alias,
             '',
-            time,
+            true,
         ]));
     }
 
@@ -322,7 +309,7 @@ const UserDetails = GObject.registerClass({
     }
 });
 
-const UserPopover = GObject.registerClass({
+export const UserPopover = GObject.registerClass({
     Template: 'resource:///org/gnome/Polari/ui/user-popover.ui',
     InternalChildren: [
         'nickLabel',
@@ -343,8 +330,6 @@ const UserPopover = GObject.registerClass({
 
         super._init(params);
 
-        this._nickLabel.set_state_flags(Gtk.StateFlags.LINK, false);
-
         this._app = Gio.Application.get_default();
 
         this._roomStatusChangedId = 0;
@@ -354,8 +339,6 @@ const UserPopover = GObject.registerClass({
         this.connect('destroy', () => {
             this.nickname = null;
         });
-
-        this.show();
     }
 
     set nickname(nickname) {
@@ -423,7 +406,13 @@ const UserPopover = GObject.registerClass({
             label = _('Offline');
         this._statusLabel.label = label;
 
-        this._nickLabel.sensitive = status === Tp.ConnectionPresenceType.AVAILABLE;
+        if (status === Tp.ConnectionPresenceType.AVAILABLE) {
+            this._nickLabel.remove_css_class('polari-inactive-nick');
+            this._nickLabel.add_css_class('polari-active-nick');
+        } else {
+            this._nickLabel.remove_css_class('polari-active-nick');
+            this._nickLabel.add_css_class('polari-inactive-nick');
+        }
     }
 
     _updateDetailsContact() {
@@ -466,7 +455,7 @@ class UserListRow extends Gtk.ListBoxRow {
 
     _createWidget() {
         let vbox = new Gtk.Box({ orientation: Gtk.Orientation.VERTICAL });
-        this.add(vbox);
+        this.set_child(vbox);
 
         let hbox = new Gtk.Box({
             margin_end: 12,
@@ -477,7 +466,7 @@ class UserListRow extends Gtk.ListBoxRow {
         });
         this._arrow = new Gtk.Image({
             icon_name: 'pan-end-symbolic',
-            no_show_all: true,
+            visible: false,
         });
         this._label = new Gtk.Label({
             label: this._user.alias,
@@ -487,14 +476,12 @@ class UserListRow extends Gtk.ListBoxRow {
             ellipsize: Pango.EllipsizeMode.END,
             max_width_chars: MAX_USERS_WIDTH_CHARS,
         });
-        hbox.add(this._label);
-        hbox.add(this._arrow);
-        vbox.add(hbox);
+        hbox.append(this._label);
+        hbox.append(this._arrow);
+        vbox.append(hbox);
 
         this._revealer = new Gtk.Revealer({ reveal_child: false });
-        vbox.add(this._revealer);
-
-        this.show_all();
+        vbox.append(this._revealer);
     }
 
     _ensureDetails() {
@@ -505,7 +492,7 @@ class UserListRow extends Gtk.ListBoxRow {
 
         this._revealer.bind_property('reveal-child', details, 'expanded', 0);
 
-        this._revealer.add(details);
+        this._revealer.set_child(details);
     }
 
     shouldShow() {
@@ -544,10 +531,10 @@ class UserListRow extends Gtk.ListBoxRow {
 
     _onExpandedChanged() {
         if (this._revealer.reveal_child) {
-            this.get_style_context().add_class('expanded');
+            this.add_css_class('expanded');
             this._arrow.icon_name = 'pan-down-symbolic';
         } else {
-            this.get_style_context().remove_class('expanded');
+            this.remove_css_class('expanded');
             this._arrow.icon_name = 'pan-end-symbolic';
             this._updateArrowVisibility();
         }
@@ -559,14 +546,14 @@ class UserList extends Gtk.ScrolledWindow {
     _init(room) {
         super._init({
             hexpand: true,
-            shadow_type: Gtk.ShadowType.ETCHED_IN,
+            has_frame: true,
             hscrollbar_policy: Gtk.PolicyType.NEVER,
             propagate_natural_height: true,
             propagate_natural_width: true,
         });
 
         this._list = new Gtk.ListBox({ vexpand: true });
-        this.add(this._list);
+        this.set_child(this._list);
         let placeholder = new Gtk.Box({
             halign: Gtk.Align.CENTER,
             valign: Gtk.Align.CENTER,
@@ -576,19 +563,16 @@ class UserList extends Gtk.ScrolledWindow {
             margin_start: 32,
             margin_end: 32,
             spacing: 6,
-            visible: true,
         });
-        placeholder.add(new Gtk.Image({
+        placeholder.append(new Gtk.Image({
             icon_name: 'edit-find-symbolic',
             pixel_size: 64,
-            visible: true,
         }));
-        placeholder.add(new Gtk.Label({
+        placeholder.append(new Gtk.Label({
             label: _('No Results'),
-            visible: true,
         }));
 
-        placeholder.get_style_context().add_class('placeholder');
+        placeholder.add_css_class('placeholder');
 
         this._list.set_placeholder(placeholder);
 
@@ -631,8 +615,6 @@ class UserList extends Gtk.ScrolledWindow {
             this._roomSignals.push(room.connect(signal.name, signal.handler));
         });
         this._onChannelChanged(room);
-
-        this.show_all();
     }
 
     get numRows() {
@@ -663,7 +645,10 @@ class UserList extends Gtk.ScrolledWindow {
     }
 
     _onChannelChanged(room) {
-        this._list.foreach(w => w.destroy());
+        [...this._list].forEach(w => {
+            this._list.remove(w);
+            w.run_dispose();
+        });
         this._rows.clear();
 
         if (!room.channel)
@@ -677,13 +662,14 @@ class UserList extends Gtk.ScrolledWindow {
     _addMember(member) {
         let row = new UserListRow(member);
         this._rows.set(member, row);
-        this._list.add(row);
+        this._list.append(row);
     }
 
     _removeMember(member) {
         let row = this._rows.get(member);
         if (row)
-            row.destroy();
+            this._list.remove(row);
+        row?.run_dispose();
         this._rows.delete(member);
     }
 
