@@ -1,16 +1,13 @@
-export { UserStatusMonitor };
-
 import Gio from 'gi://Gio';
 import GLib from 'gi://GLib';
 import GObject from 'gi://GObject';
 import Polari from 'gi://Polari';
 import Tp from 'gi://TelepathyGLib';
 
-import { AccountsMonitor } from './accountsMonitor.js';
-import { RoomManager } from './roomManager.js';
-import * as Utils from './utils.js';
+import AccountsMonitor from './accountsMonitor.js';
+import RoomManager from './roomManager.js';
 
-const UserStatusMonitor = class {
+export default class UserStatusMonitor {
     static getDefault() {
         if (!this._singleton)
             this._singleton = new UserStatusMonitor();
@@ -62,7 +59,7 @@ const UserStatusMonitor = class {
     getUserTrackerForAccount(account) {
         return this._userTrackers.get(account);
     }
-};
+}
 
 
 const UserTracker = GObject.registerClass({
@@ -371,12 +368,12 @@ const UserTracker = GObject.registerClass({
     _notifyNickAvailable(member, room) {
         let notification = new Gio.Notification();
         notification.set_title(_('User is online'));
-        notification.set_body(_('User %s is now online.').format(member.alias));
+        vprintf(notification.set_body(_('User %s is now online.'), member.alias));
 
-        let param = GLib.Variant.new('(ssu)', [
+        let param = GLib.Variant.new('(ssb)', [
             this._account.get_object_path(),
             room.channel_name,
-            Utils.getTpEventTime(),
+            true,
         ]);
         notification.set_default_action_and_target('app.join-room', param);
 
