@@ -20,39 +20,50 @@
  *
  */
 
-export {
-    isFlatpakSandbox,
-    touchFile,
-    needsOnetimeAction,
-    getTpEventTime,
-    findUrls,
-    findChannels,
-    openURL,
-    updateTerms,
-    gpaste,
-    imgurPaste,
-    storeAccountPassword,
-    storeIdentifyPassword,
-    lookupAccountPassword,
-    lookupIdentifyPassword,
-    clearAccountPassword,
-    clearIdentifyPassword,
-    formatTimePassed
-};
-
 import Gdk from 'gi://Gdk';
 import Gio from 'gi://Gio';
 import GLib from 'gi://GLib';
 import Gtk from 'gi://Gtk';
 import Secret from 'gi://Secret';
-import Soup from 'gi://Soup?version=2.4';
-import Tp from 'gi://TelepathyGLib';
 
-import * as AppNotifications from './appNotifications.js';
+import { SimpleOutput as AppNotification } from './appNotifications.js';
+
+import gi from 'gi';
+let Soup;
+
+try {
+    Soup = gi.require('Soup', '3.0');
+} catch (e) {
+    Soup = gi.require('Soup', '2.4');
+
+    Soup.Message.new_from_encoded_form =
+        function (method, uri, form) {
+            const message = Soup.Message.new_from_uri(method, new Soup.URI(uri));
+            message.set_request(
+                Soup.FORM_MIME_TYPE_URLENCODED,
+                Soup.MemoryUse.COPY,
+                form);
+            return message;
+        };
+
+    Soup.Session.prototype.send_and_read_async =
+        function (message, prio, cancellable, callback) {
+            this.queue_message(message, () => callback(this, message));
+        };
+    Soup.Session.prototype.send_and_read_finish =
+        function (message) {
+            if (message.status_code !== Soup.KnownStatusCode.OK)
+                return null;
+
+            return message.response_body.flatten().get_as_bytes();
+        };
+}
 
 Gio._promisify(Secret, 'password_store', 'password_store_finish');
 Gio._promisify(Secret, 'password_lookup', 'password_lookup_finish');
 Gio._promisify(Secret, 'password_clear', 'password_clear_finish');
+Gio._promisify(Soup.Session.prototype,
+    'send_and_read_async', 'send_and_read_finish');
 
 const SECRET_SCHEMA_ACCOUNT = new Secret.Schema(
     'org.gnome.Polari.Account',
@@ -113,13 +124,20 @@ let _gpasteExpire;
 
 let _inFlatpakSandbox;
 
-function isFlatpakSandbox() {
+/**
+ * @returns {bool}
+ */
+export function isFlatpakSandbox() {
     if (_inFlatpakSandbox === undefined)
         _inFlatpakSandbox = GLib.file_test('/.flatpak-info', GLib.FileTest.EXISTS);
     return _inFlatpakSandbox;
 }
 
-function touchFile(file) {
+/**
+ * @param {Gio.File} file - file to touch
+ * @throws
+ */
+export function touchFile(file) {
     try {
         file.get_parent().make_directory_with_parents(null);
     } catch (e) {
@@ -132,7 +150,11 @@ function touchFile(file) {
     stream.close(null);
 }
 
-function needsOnetimeAction(name) {
+/**
+ * @param {string} name - name of the one-time action
+ * @returns {bool}
+ */
+export function needsOnetimeAction(name) {
     let path = GLib.build_filenamev([
         GLib.get_user_data_dir(),
         'polari',
@@ -144,25 +166,30 @@ function needsOnetimeAction(name) {
     } catch (e) {
         if (e.matches(Gio.IOErrorEnum, Gio.IOErrorEnum.EXISTS))
             return false;
-        log(`Failed to mark onetime action ${name} as completed: ${e.message}`);
+        console.warn(`Failed to mark onetime action ${
+            name} as completed`);
+        console.debug(e);
     }
     return true;
 }
 
-function getTpEventTime() {
-    let time = Gtk.get_current_event_time();
-    if (time === 0)
-        return GLib.MAXUINT32;
-    return Tp.user_action_time_from_x11(time);
-}
-
-function storeAccountPassword(account, password) {
-    let label = _('Polari server password for %s').format(account.display_name);
+/**
+ * @param {Tp.Account} account - the account
+ * @param {string} password - the password
+ * @throws
+ */
+export function storeAccountPassword(account, password) {
+    let label = vprintf(_('Polari server password for %s'), account.display_name);
     _storePassword(SECRET_SCHEMA_ACCOUNT, label, account, password);
 }
 
-function storeIdentifyPassword(account, password) {
-    let label = _('Polari NickServ password for %s').format(account.display_name);
+/**
+ * @param {Tp.Account} account - the account
+ * @param {string} password - the password
+ * @throws
+ */
+export function storeIdentifyPassword(account, password) {
+    let label = vprintf(_('Polari NickServ password for %s'), account.display_name);
     _storePassword(SECRET_SCHEMA_IDENTIFY, label, account, password);
 }
 
@@ -173,16 +200,27 @@ async function _storePassword(schema, label, account, password) {
         await Secret.password_store(schema, attr, coll, label, password, null);
     } catch (e) {
         const name = account.display_name;
-        log(`Failed to store password for account ${name}: ${e.message}`);
+        console.warn(`Failed to store password for account ${name}`);
+        console.debug(e);
         throw e;
     }
 }
 
-function lookupAccountPassword(account) {
+/**
+ * @param {Tp.Account} account - the account
+ * @returns {string} - the password
+ * @throws
+ */
+export function lookupAccountPassword(account) {
     return _lookupPassword(SECRET_SCHEMA_ACCOUNT, account);
 }
 
-function lookupIdentifyPassword(account) {
+/**
+ * @param {Tp.Account} account - the account
+ * @returns {string}
+ * @throws
+ */
+export function lookupIdentifyPassword(account) {
     return _lookupPassword(SECRET_SCHEMA_IDENTIFY, account);
 }
 
@@ -193,18 +231,27 @@ async function _lookupPassword(schema, account) {
         password = await Secret.password_lookup(schema, attr, null);
     } catch (e) {
         const name = account.display_name;
-        log(`Failed to lookup password for account "${name}": ${e.message}`);
+        console.warn(`Failed to lookup password for account ${name}`);
+        console.debug(e);
         throw e;
     }
 
     return password;
 }
 
-function clearAccountPassword(account) {
+/**
+ * @param {Tp.Account} account - the account
+ * @throws
+ */
+export function clearAccountPassword(account) {
     _clearPassword(SECRET_SCHEMA_ACCOUNT, account);
 }
 
-function clearIdentifyPassword(account) {
+/**
+ * @param {Tp.Account} account - the account
+ * @throws
+ */
+export function clearIdentifyPassword(account) {
     _clearPassword(SECRET_SCHEMA_IDENTIFY, account);
 }
 
@@ -214,20 +261,24 @@ async function _clearPassword(schema, account) {
         await Secret.password_clear(schema, attr, null);
     } catch (e) {
         const name = account.display_name;
-        log(`Failed to clear password for account "${name}": ${e.message}`);
+        console.warn(`Failed to clear password for account ${name}`);
+        console.debug(e);
         throw e;
     }
 }
 
-// findUrls:
-// @str: string to find URLs in
-//
-// Searches @str for URLs and returns an array of objects with %url
-// properties showing the matched URL string, and %pos properties indicating
-// the position within @str where the URL was found.
-//
-// Return value: the list of match objects, as described above
-function findUrls(str) {
+/**
+ * @typedef {Object} UrlMatch
+ * @property {string} url - the matched URL
+ * @property {string} name - the user-visible name of the URL
+ * @property {number} pos - the position of the match
+ */
+
+/**
+ * @param {string} str - string to find URLs in
+ * @returns {UrlMatch}
+ */
+export function findUrls(str) {
     let res = [], match;
     while ((match = _urlRegexp.exec(str))) {
         let name = match[2];
@@ -237,7 +288,12 @@ function findUrls(str) {
     return res;
 }
 
-function findChannels(str, server) {
+/**
+ * @param {string} str - string to find IRC channel references in
+ * @param {string} server - server to use in channel URLs
+ * @returns {UrlMatch}
+ */
+export function findChannels(str, server) {
     let res = [], match;
     while ((match = _channelRegexp.exec(str))) {
         res.push({
@@ -249,21 +305,30 @@ function findChannels(str, server) {
     return res;
 }
 
-function openURL(url, timestamp) {
+/**
+ * @param {string} url - the url
+ */
+export function openURL(url) {
     let app = Gio.Application.get_default();
-    try {
-        if (app.active_window)
-            Gtk.show_uri_on_window(app.active_window, url, timestamp);
-        else
-            Gtk.show_uri(Gdk.Screen.get_default(), url, timestamp);
-    } catch (e) {
-        let n = new AppNotifications.SimpleOutput(_('Failed to open link'));
-        app.notificationQueue.addNotification(n);
-        debug(`Failed to open ${url}: ${e.message}`);
-    }
+    Gtk.show_uri_full(
+        app.active_window, url, Gdk.CURRENT_TIME, null,
+        (o, res) => {
+            try {
+                Gtk.show_uri_full_finish(app.active_window, res);
+            } catch (e) {
+                let n = new AppNotification(_('Failed to open link'));
+                app.active_window?.queueNotification(n);
+                console.debug(`Failed to open ${url}: %o`, e);
+            }
+        });
 }
 
-function updateTerms(terms, str) {
+/**
+ * @param {string[]} terms - list of search terms (words) to update
+ * @param {string} str - search string
+ * @returns {bool} - whether terms changed
+ */
+export function updateTerms(terms, str) {
     let normalized = str.trim().toLowerCase().replace(/\s+/g, ' ');
     let newTerms = normalized ? normalized.split(' ') : [];
 
@@ -277,24 +342,17 @@ function updateTerms(terms, str) {
     return changed;
 }
 
-function _queueSoupMessage(session, message) {
-    return new Promise((resolve, reject) => {
-        session.queue_message(message, () => {
-            const { statusCode } = message;
-            if (statusCode === Soup.KnownStatusCode.OK)
-                resolve(message.responseBody.data);
-            else
-                reject(new Error(`Got unexpected response ${statusCode}`));
-        });
-    });
-}
-
 async function _getGpasteExpire() {
-    let session = new Soup.Session();
-    let paramUrl = `${GPASTE_BASEURL}api/json/parameter/expire`;
-    let message = Soup.form_request_new_from_hash('GET', paramUrl, {});
+    const session = new Soup.Session();
+    const message = Soup.Message.new('GET',
+        `${GPASTE_BASEURL}api/json/parameter/expire`);
 
-    const json = await _queueSoupMessage(session, message);
+    const bytes = await session.send_and_read_async(
+        message,
+        GLib.PRIORITY_DEFAULT,
+        null);
+    checkResponse(message);
+    const json = new TextDecoder().decode(bytes.get_data());
     const info = JSON.parse(json);
 
     const values = info.result?.values;
@@ -307,7 +365,13 @@ async function _getGpasteExpire() {
     }, 0).toString();
 }
 
-async function gpaste(text, title) {
+/**
+ * @param {string} text - text to upload
+ * @param {string} title - title to use
+ * @throws
+ * @returns {string} - the paste URL
+ */
+export async function gpaste(text, title) {
     if (_gpasteExpire === undefined)
         _gpasteExpire = await _getGpasteExpire();
 
@@ -321,11 +385,17 @@ async function gpaste(text, title) {
         language: 'text',
     };
 
-    let session = new Soup.Session();
-    let createUrl = `${GPASTE_BASEURL}api/json/create`;
-    let message = Soup.form_request_new_from_hash('POST', createUrl, params);
+    const session = new Soup.Session();
+    const message = Soup.Message.new_from_encoded_form('POST',
+        `${GPASTE_BASEURL}api/json/create`,
+        Soup.form_encode_hash(params));
 
-    const json = await _queueSoupMessage(session, message);
+    const bytes = await session.send_and_read_async(
+        message,
+        GLib.PRIORITY_DEFAULT,
+        null);
+    checkResponse(message);
+    const json = new TextDecoder().decode(bytes.get_data());
     const info = JSON.parse(json);
 
     if (!info.result?.id)
@@ -333,7 +403,13 @@ async function gpaste(text, title) {
     return `${GPASTE_BASEURL}${info.result.id}`;
 }
 
-async function imgurPaste(pixbuf, title) {
+/**
+ * @param {GdkPixbuf.Pixbuf} pixbuf - the pixbuf to paste
+ * @param {string} title - the title to use
+ * @returns {string} - the paste URL
+ * @throws
+ */
+export async function imgurPaste(pixbuf, title) {
     let [success, buffer] = pixbuf.save_to_bufferv('png', [], []);
     if (!success)
         throw new Error('Failed to create image buffer');
@@ -343,14 +419,20 @@ async function imgurPaste(pixbuf, title) {
         image: GLib.base64_encode(buffer),
     };
 
-    let session = new Soup.Session();
-    let createUrl = 'https://api.imgur.com/3/image';
-    let message = Soup.form_request_new_from_hash('POST', createUrl, params);
+    const session = new Soup.Session();
+    const message = Soup.Message.new_from_encoded_form('POST',
+        'https://api.imgur.com/3/image',
+        Soup.form_encode_hash(params));
 
     let requestHeaders = message.request_headers;
     requestHeaders.append('Authorization', `Client-ID ${IMGUR_CLIENT_ID}`);
 
-    const json = await _queueSoupMessage(session, message);
+    const bytes = await session.send_and_read_async(
+        message,
+        GLib.PRIORITY_DEFAULT,
+        null);
+    checkResponse(message);
+    const json = new TextDecoder().decode(bytes.get_data());
     const info = JSON.parse(json);
 
     if (!info.success)
@@ -359,7 +441,18 @@ async function imgurPaste(pixbuf, title) {
     return info.data.link;
 }
 
-function formatTimePassed(seconds) {
+function checkResponse(message) {
+    const { statusCode } = message;
+    const phrase = Soup.Status.get_phrase(statusCode);
+    if (statusCode !== Soup.Status.OK)
+        throw new Error(`Unexpected response: ${phrase}`);
+}
+
+/**
+ * @param {number} seconds - time in seconds
+ * @returns {string}
+ */
+export function formatTimePassed(seconds) {
     if (seconds === 0)
         return _('Now');
 
@@ -367,41 +460,41 @@ function formatTimePassed(seconds) {
         return _('Unavailable');
 
     if (seconds < 60) {
-        return ngettext(
+        return vprintf(ngettext(
             '%d second ago',
-            '%d seconds ago', seconds).format(seconds);
+            '%d seconds ago', seconds), seconds);
     }
 
     let minutes = seconds / 60;
     if (minutes < 60) {
-        return ngettext(
+        return vprintf(ngettext(
             '%d minute ago',
-            '%d minutes ago', minutes).format(minutes);
+            '%d minutes ago', minutes), minutes);
     }
 
     let hours = minutes / 60;
     if (hours < 24) {
-        return ngettext(
+        return vprintf(ngettext(
             '%d hour ago',
-            '%d hours ago', hours).format(hours);
+            '%d hours ago', hours), hours);
     }
 
     let days = hours / 24;
     if (days < 7) {
-        return ngettext(
+        return vprintf(ngettext(
             '%d day ago',
-            '%d days ago', days).format(days);
+            '%d days ago', days), days);
     }
 
     let weeks = days / 7;
     if (days < 30) {
-        return ngettext(
+        return vprintf(ngettext(
             '%d week ago',
-            '%d weeks ago', weeks).format(weeks);
+            '%d weeks ago', weeks), weeks);
     }
 
     let months = days / 30;
-    return ngettext(
+    return vprintf(ngettext(
         '%d month ago',
-        '%d months ago', months).format(months);
+        '%d months ago', months), months);
 }
